/*************************************************************
Module		: LPC driver module
Modified by	: Connie Yin  on 02/10/2007
*************************************************************/
/* Version Number of this module */
#define SAMPLE_MAJOR	1
#define SAMPLE_MINOR	0

#define SUPPORT_BBSP
#define SUPPORT_PORT80

#include <linux/module.h>
#include <linux/kernel.h>	/* printk() 		*/
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/ide.h>

#define	dbgprint	if(Debug_Enable) printk

#include "kcs.h"
#define KCS_REG_CHR_DEV_FAILS 		0xAA
#define KCS_REGIS_IRQ_FAILS			0xDD
#define SET_KCS_OPTION				1

int  (*ResetFunction)(void) = NULL;

static int kcs_irq_requested = 0;

/*** Prototype Declaration ***/
static void KCSRcvByte (u8 ChannelNum);

#define TRANSMIT_TIMEOUT	10000	/* Transmission Timeout in msecs */	

/* VSC452 Hardware interrupt forcing register */
#define HIF_REG         (volatile unsigned long *)0xB02008f4


/* VSC452 Interrupt observe registers */
#define IOR0_REG        (volatile unsigned long *)0xB02008f8
#define IOR1_REG        (volatile unsigned long *)0xB02008fc


/* SIO Registers    */
#define SIO_INDEX_REG	(volatile unsigned long*)0xB03000B8
#define SIO_DATA_REG	(volatile unsigned long*)0XB03000BC 

/* VSC452 KCS interrupt masks coreeponding to observer registers   ( bits  22,23,24,25 )*/
#define KCS0_INTMASK		0x00400000
#define KCS1_INTMASK		0x00800000

/* BBSP and the fourth KCS*/
#define KCS2_INTMASK		0x01000000
#define KCS3_INTMASK		0x02000000

/* KCS Intr IBF and OBF enable mask*/
#define INTR_EN_MASK		0x06
#define INTR_IBF_EN_MASK	0x02


#ifdef SUPPORT_BBSP
/*BBSP Channel*/
#define BBSP_CHANNEL  3
#endif

#ifdef SUPPORT_PORT80
#define PORT80_CHANNEL	4
#endif
/* IOCTL related */
#define	ARBITRARY_NUM		100
#define MEL_IOCTL_TEST		_IOWR(ARBITRARY_NUM, 0, int)
#define MEL_IOCTL_READ		_IOR(ARBITRARY_NUM, 1, struct ioctl_t *)
#define MEL_IOCTL_WRITE		_IOW(ARBITRARY_NUM, 4, struct ioctl_t *)


static int KCS_MAJOR = 42;
static const char KCS_DEVNAME[] = "KCS";


#ifdef SUPPORT_BBSP
/* flag to indicate bbsp data change*/
static u8 is_new_bbspdata = 0;
#endif
	


/* semaphore to provide mutex */
struct semaphore g_sem;




/* Prototypes  */
static void SendKCSResponse (u8 ChannelNum, IPMICmdMsg_T *pKCSCmdMsg);

static int 	init_kcs_module(void);
static void cleanup_kcs_module(void);

static ssize_t kcs1_read(struct file * filp, char * buf,size_t count,loff_t *ppos);
static ssize_t kcs2_read(struct file * filp, char * buf,size_t count,loff_t *ppos);


static ssize_t kcs1_write (struct file *file, const char *buf, size_t count,loff_t *offset);
static ssize_t kcs2_write (struct file *file, const char *buf, size_t count,loff_t *offset);


static int kcs1_release(struct inode * inode, struct file * filp);
static int kcs2_release(struct inode * inode, struct file * filp);


static  ssize_t bbsp_read(struct file * file, char * buf,size_t count,loff_t *ppos);
static ssize_t bbsp_write(struct file *file, const char *buf, size_t count,loff_t *offset);
#ifdef SUPPORT_PORT80
static  ssize_t port80_read(struct file * file, char * buf,size_t count,loff_t *ppos);
static ssize_t port80_write(struct file *file, const char *buf, size_t count,loff_t *offset);
static int port80_release(struct inode * inode, struct file * filp);
static char port80_readreg(void);
static int port80_writereg(char data);
#endif
static char bbsp_readreg(void);
static int bbsp_writereg(char data);

static int bbsp_release(struct inode * inode, struct file * filp);

static int kcs_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg);

void Kcs_OsInitSleepStruct(Kcs_OsSleepStruct *Sleep);
void Kcs_OsWakeupOnTimeout(Kcs_OsSleepStruct *Sleep);
long Kcs_OsSleepOnTimeout(Kcs_OsSleepStruct *Sleep,u8 *Var,long msecs);
static void SleepTimeOut(unsigned long SleepPtr);
static int KcsRequest(char * buf , u8 ChannelNum);
static int kcs_open(struct inode * inode, struct file * filp);
static size_t KcsWriteRequest (const char *buf, size_t count, u8 ChannelNum );



static struct file_operations kcs_fops = {
	owner:		THIS_MODULE,
	read:		NULL,
	write:		NULL,
	ioctl:		kcs_ioctl,
	open:		kcs_open,
	release:	NULL,
};

static struct file_operations kcs1_fops = {
	owner:		THIS_MODULE,
	read:		kcs1_read,
	write:		kcs1_write,
	ioctl:		kcs_ioctl,
	open:		NULL,
	release:	kcs1_release,
};

static struct file_operations kcs2_fops = {
	owner:		THIS_MODULE,
	read:		kcs2_read,
	write:		kcs2_write,
	ioctl:		kcs_ioctl,
	open:		NULL,
	release:	kcs2_release,
};

static struct file_operations kcs3_fops = {
        owner:          THIS_MODULE,
        read:          	bbsp_read,
        write:          bbsp_write,
        ioctl:          kcs_ioctl,
        open:           NULL,
        release:        bbsp_release,
};
#ifdef SUPPORT_PORT80
static struct file_operations kcs4_fops = {
        owner:          THIS_MODULE,
        read:          	port80_read,
        write:          port80_write,
        ioctl:          kcs_ioctl,
        open:           NULL,
        release:        port80_release,
};
#endif
extern void HandleSnoopReset(void);
extern int write_init_st(int option,int value);

static KCSBuf_T		m_KCSBuf [MAX_KCS_CHANNELS];
static IPMICmdMsg_T   m_KCS1CmdMsg;
static IPMICmdMsg_T   m_KCS2CmdMsg; 
static IPMICmdMsg_T   m_KCS3CmdMsg; 
static IPMICmdMsg_T   m_KCS4CmdMsg;
static short init_kcs_status=0;

volatile u32 * kcs1_vadd;
volatile u32 * kcs2_vadd;
volatile u32 * kcs3_vadd;
volatile u32 * kcs4_vadd;

static u8 sg_u8LPCReset = 0;

struct timer_list KcsTimer;
#define POLL_MSEC HZ

#if defined(SUPPORT_BBSP) || defined (SUPPORT_PORT80)
extern struct class * get_hwmon_class(void);
#endif
#ifdef SUPPORT_BBSP
/** sysfs  data structures **/
static ssize_t showbbsp(struct class *,  char *buffer);
static ssize_t storebbsp(struct class *,  const char *buffer, size_t count);

static CLASS_ATTR(bbsp,0666,showbbsp,storebbsp);
struct class * cls = NULL;

/*show bbsp*/
static ssize_t showbbsp(struct class *cls,  char *buffer)
{
	u8 data;
	
	data = bbsp_readreg();
	return sprintf(buffer, "%02x\n", data);
}

/*store bbsp*/
static ssize_t storebbsp(struct class *kobj,  const char *buffer, size_t count)
{
	unsigned int 		val =0 ;

	if ( 0 == count )		/* return if count 0*/
		return 0;

	if ( NULL == buffer )
		return count;
	sscanf ( buffer, "%2x", &val);
	bbsp_writereg(val);
	return count;
}

/** syfs ends **/
#endif

#ifdef SUPPORT_PORT80
/** sysfs  data structures **/
static ssize_t showport80(struct class *,  char *buffer);
static ssize_t storeport80(struct class *,  const char *buffer, size_t count);

static CLASS_ATTR(port80,0666,showport80,storeport80);
struct class * port80cls = NULL;
/*show port80*/
static ssize_t showport80(struct class *cls,  char *buffer)
{
	u8 data;
	data = port80_readreg();
	return sprintf(buffer, "%x\n", data);
}

/*store port80*/
static ssize_t storeport80(struct class *kobj,  const char *buffer, size_t count)
{
	unsigned int val =0 ;
	if ( 0 == count )		/* return if count 0*/
		return 0;
	if ( NULL == buffer )
		return count;
	sscanf ( buffer, "%2x", &val);
	port80_writereg(val);
	printk(KERN_ALERT "Value stores in port 0x80 is %x\n", val);
	return count;
}

/** syfs ends **/
#endif

static u8	Debug_Enable=0;
/*--------------------- If Module define the entry and exit  ------------------*/
#ifdef MODULE


void sio_registers_init(void)
{
	/* KCS Channel 1 */
	*(SIO_INDEX_REG)= 0x07;
	*(SIO_DATA_REG) = 0x07;		/*Select KCS0 */
	*(SIO_INDEX_REG)= 0x60;		/* Base Addr */
	*(SIO_DATA_REG) = 0x0C;
	*(SIO_INDEX_REG)= 0x61;
	*(SIO_DATA_REG) = 0xA2;
	*(SIO_INDEX_REG)= 0x70;
        *(SIO_DATA_REG) = 0x00;
	*(SIO_INDEX_REG)= 0x30;		/*Activate the channel*/	
	*(SIO_DATA_REG) = 0x01;	
	
	 /* KCS Channel 2 */
        *(SIO_INDEX_REG)= 0x07;
        *(SIO_DATA_REG) = 0x08;         /*Select KCS1 */
        *(SIO_INDEX_REG)= 0x60;         /* Base Addr */
        *(SIO_DATA_REG) = 0x0C;
        *(SIO_INDEX_REG)= 0x61;
        *(SIO_DATA_REG) = 0xA4;
        *(SIO_INDEX_REG)= 0x70;
        *(SIO_DATA_REG) = 0x00;
        *(SIO_INDEX_REG)= 0x30;         /*Activate the channel*/
        *(SIO_DATA_REG) = 0x01;

#ifdef SUPPORT_BBSP
	 /* BBSP Channel 1 */
        *(SIO_INDEX_REG)= 0x07;
        *(SIO_DATA_REG) = 0x09;         /*Select KCS2 */
        *(SIO_INDEX_REG)= 0x60;         /* Base Addr */
        *(SIO_DATA_REG) = 0x01;
        *(SIO_INDEX_REG)= 0x61;
        *(SIO_DATA_REG) = 0x80;
        *(SIO_INDEX_REG)= 0x70;
        *(SIO_DATA_REG) = 0x00;
        *(SIO_INDEX_REG)= 0x30;         /*Activate the channel*/
        *(SIO_DATA_REG) = 0x01;
#endif

}

static void process_kcs_intr (void)
{
    u8  ChannelNum;
    u8  Status =0;
    u8 int_flag = 0;

    for (ChannelNum=1; ChannelNum <= MAX_KCS_CHANNELS; ChannelNum++)
    {
		READ_KCS_STATUS_REG (ChannelNum, Status);

		if (IBF_BIT_SET(Status))
		{
			KCSRcvByte(ChannelNum);
			CLEAR_IBF_STATUS(ChannelNum);
			int_flag = 1;
		//	break;
		}
    }

    if (int_flag == 0)
    {
		//printk(KERN_ERR ":SI ");
	    ///printk(KERN_ERR "Error : Unknown channel\n");
    }

    return;
}


static irqreturn_t lpc_handler ( int irq, void *dev_id)
{	
	unsigned long ior0_val,ior1_val;

	/* VSC452 Specific check to make sure KCS specific interrupts triggered */

	 ior0_val = *IOR0_REG;
	 ior1_val = *IOR1_REG;
	 *HIF_REG = 0; 

	 if ( !(ior0_val & KCS0_INTMASK) 
	 	  && !(ior0_val & KCS1_INTMASK)
		  && !(ior0_val & KCS2_INTMASK)	&& !(ior0_val & KCS3_INTMASK)	) 
	 {
		dbgprint(KERN_ALERT "interrupt not belongs us\n");
		return IRQ_NONE;
	 }		 

	dbgprint(KERN_ALERT "\n Inside the Handler\n");

	process_kcs_intr();
	return (IRQ_HANDLED);
}

static int
init_kcs_module(void)
{
	int 	err, ret = 0;
	u8	ChannelNum ;
	u8	Status =0;
	unsigned long *va_base;

	KCSBuf_T*		pKCSBuf1;
	KCSBuf_T*		pKCSBuf2;
	KCSBuf_T*		pKCSBuf3;
	KCSBuf_T*		pKCSBuf4;

	pKCSBuf1 = &m_KCSBuf [KCS_1_CHANNEL];
	pKCSBuf2 = &m_KCSBuf [KCS_2_CHANNEL];
	pKCSBuf3 = &m_KCSBuf [KCS_3_CHANNEL];
	pKCSBuf4 = &m_KCSBuf [KCS_4_CHANNEL];


	printk(KERN_ALERT "LPC/KCS/Port80 Driver Version %d.%d\n",SAMPLE_MAJOR,SAMPLE_MINOR);
	printk(KERN_ALERT "Copyright (c) 2007 American Megatrends Inc.\n");

	/* ----  Register the character device ------------------- */
	if ((err = register_chrdev(KCS_MAJOR, KCS_DEVNAME, &kcs_fops)) < 0)
	{
		printk (KERN_ERR "failed to register device (err %d)\n", err);
		init_kcs_status=KCS_REG_CHR_DEV_FAILS;
		ret = -ENODEV;
		goto fail;
	}

	kcs1_vadd = (u32*)KCS0_BASE;
	kcs2_vadd = (u32*)KCS1_BASE;
	kcs3_vadd = (u32*)KCS2_BASE;
	kcs4_vadd = (u32*)KCS3_BASE;

      /* Init KCS */
	m_KCSBuf [KCS_1_CHANNEL].pKCSRcvPkt = m_KCS1CmdMsg.Request;
	m_KCSBuf [KCS_2_CHANNEL].pKCSRcvPkt = m_KCS2CmdMsg.Request;
	m_KCSBuf [KCS_3_CHANNEL].pKCSRcvPkt = m_KCS3CmdMsg.Request;
	m_KCSBuf [KCS_4_CHANNEL].pKCSRcvPkt = m_KCS4CmdMsg.Request;
	
	//Clear status
	for (ChannelNum=1; ChannelNum <= MAX_KCS_CHANNELS ; ChannelNum++)
	{
		READ_KCS_STATUS_REG (ChannelNum, Status);

		if (IBF_BIT_SET(Status))
		{
			KCSRcvByte(ChannelNum);
			CLEAR_IBF_STATUS(ChannelNum);
		}
	}

	

	pKCSBuf1->KcsWakeup = 0;
	pKCSBuf2->KcsWakeup = 0;
	pKCSBuf3->KcsWakeup = 0;
	pKCSBuf4->KcsWakeup = 0;

	pKCSBuf1->TxReady = 0;
	pKCSBuf2->TxReady = 0;
	pKCSBuf3->TxReady = 0;
	pKCSBuf4->TxReady = 0;

	pKCSBuf1->KcsIFActive = 0;
	pKCSBuf2->KcsIFActive = 0;
	pKCSBuf3->KcsIFActive = 0;
	pKCSBuf4->KcsIFActive = 0;

	pKCSBuf1->KcsWtIFActive = 0;
	pKCSBuf2->KcsWtIFActive = 0;
	pKCSBuf3->KcsWtIFActive = 0;
	pKCSBuf4->KcsWtIFActive = 0;



	pKCSBuf1->FirstTime = 1;
	pKCSBuf2->FirstTime = 1;
	pKCSBuf3->FirstTime = 1;
	pKCSBuf4->FirstTime = 1;	

	pKCSBuf1->KcsResFirstTime = 1;
	pKCSBuf2->KcsResFirstTime = 1;
	pKCSBuf3->KcsResFirstTime = 1;
	pKCSBuf4->KcsResFirstTime = 1;
	
	pKCSBuf1->KcsINuse = 0;
	pKCSBuf2->KcsINuse = 0;
	pKCSBuf3->KcsINuse = 0;
	pKCSBuf4->KcsINuse = 0;


	if ((err = request_irq(IRQ_KCS_BT_SMIC_MAIL, lpc_handler, IRQF_DISABLED, "KCS", 0)) < 0)
	{
		printk(KERN_ERR "failed to request irq %d (err %d)\n", IRQ_KCS_BT_SMIC_MAIL, err);
		init_kcs_status=KCS_REGIS_IRQ_FAILS;
	    	ret = -EBUSY;
		goto fail;
	}
	kcs_irq_requested = 1;

	/*Enable Interrupts for KCS1 */
	va_base = (unsigned long *)KCS0_BASE; //ioremap_nocache( KCS0_BASE , 512);
	
	/* Clear IBF and OBF  Flags  */
	*(va_base+STATUS_REG) = 0x00000000;

	/* Enable LPC Interrupt */
	*(va_base+MODE) = INTR_EN_MASK;

	
	/*Enable Interrupts for KCS2 */
	va_base = (unsigned long *)KCS1_BASE; //ioremap_nocache( KCS0_BASE , 512);
	
	/* Clear IBF and OBF  Flags  */
	*(va_base+STATUS_REG) = 0x00000000;

	/* Enable LPC Interrupt */
	*(va_base+MODE) = INTR_EN_MASK;
	
	/* Enable Interrupts fpr BBSP */	
	va_base = (unsigned long *)KCS2_BASE; //ioremap_nocache( KCS0_BASE , 512);

	/* Clear IBF and OBF Flags */
	*(va_base+STATUS_REG) = 0x00000000;
	
	/*Enable LPC interupt */
	*(va_base+MODE) = INTR_EN_MASK;
#ifdef SUPPORT_BBSP
	init_MUTEX(&g_sem);
	/*regiser hooks for sysfs*/
	cls = get_hwmon_class();
	if( cls )
	{
		err = class_create_file(cls, &class_attr_bbsp);	
		if (err)
			printk (KERN_ALERT "kcs: failed to create hwmon class file");
	}
#endif
#ifdef SUPPORT_PORT80
	/* Enable Interrupts fpr PORT80 */
	va_base = (unsigned long *)KCS3_BASE; //ioremap_nocache( KCS0_BASE , 512);
	
	/* Clear IBF and OBF Flags */
	*(va_base+STATUS_REG) = 0x00000000;
	
	/*Enable LPC interupt */
	*(va_base+MODE) = INTR_EN_MASK;
	
	port80cls = get_hwmon_class();
	if( port80cls )
	{
		err = class_create_file(port80cls, &class_attr_port80);
		if (err)
			printk(KERN_ALERT "KCS: Failed to create port80 sysfs file\n");
	}
#endif

	/*SIO register Initialization using Backdoor access*/
	//sio_registers_init();
	write_init_st(SET_KCS_OPTION,init_kcs_status);			
	return ret;	/* Return 1 to not load the module */

fail:
	cleanup_kcs_module ();
	return ret;
}

static void
cleanup_kcs_module(void)
{
	printk(KERN_ALERT "Unloading LPC/KCS/Port80 Module ..............\n");
	
	if(init_kcs_status==0)
	{
		init_kcs_status=0xff;
	}
	write_init_st(SET_KCS_OPTION,init_kcs_status);

	/* free irq */
	if (kcs_irq_requested)
	{
		free_irq(IRQ_KCS_BT_SMIC_MAIL, 0);
		kcs_irq_requested = 0;
	}


	/* unregister the device */
	unregister_chrdev(KCS_MAJOR, KCS_DEVNAME);

#ifdef SUPPORT_BBSP
	
	/*sysfs cleanup*/
	if ( cls)
		class_remove_file(cls, &class_attr_bbsp);
#endif		
#ifdef SUPPORT_PORT80
	/*port80 sysfs cleanup*/
	if (port80cls)
		class_remove_file(port80cls, &class_attr_port80);
#endif		
	printk(KERN_ALERT "kcs module unloaded sucessfully.\n");

	return;
}

struct	ioctl_data_t
{
	unsigned char	address;
	unsigned char	data;
};




static int KcsRequest(char * buf , u8 ChannelNum)
{

	KCSBuf_T*		pKCSBuf;
	pKCSBuf = &m_KCSBuf [ChannelNum];
        
	dbgprint(KERN_ALERT "Inside KcsRequest ChannelNum=%x\n", ChannelNum);

	/* Initialize Sleep Structure for the First Time */
	if (pKCSBuf->FirstTime)
	{
		Kcs_OsInitSleepStruct(&(pKCSBuf->KcsReqWait));
		pKCSBuf->FirstTime = 0;
	}

  	dbgprint(KERN_ALERT "Sleeping in KcsRequest\n");
	pKCSBuf->KcsIFActive 	= 1;
	Kcs_OsSleepOnTimeout(&(pKCSBuf->KcsReqWait),&(pKCSBuf->KcsWakeup),0);
	pKCSBuf->KcsWakeup 	= 0;
	pKCSBuf->KcsIFActive 	= 0;

	dbgprint(KERN_ALERT "Out of sleep in KcsRequest\n");
	/*Copy from kernel data area to user data area*/
	__copy_to_user( (void *)buf, (void *)pKCSBuf->pKCSRcvPkt,  pKCSBuf->KCSRcvPktIx );
	
	return pKCSBuf->KCSRcvPktIx;

}

static int kcs_open(struct inode * inode, struct file * filp)
{
	
	KCSBuf_T*		pKCSBuf;
	
//	printk(KERN_ALERT "first inode->i_rdev %X" , inode->i_rdev);
	if(filp->f_flags & 010000000)
	{
		return 0;
	}
//	printk(KERN_ALERT "inode->i_rdev %X\n" , inode->i_rdev);
	switch (MINOR(inode->i_rdev)) {
	case 1:
		dbgprint(KERN_ALERT "kcs_open, case is 1\n");
		//save_flags(flags); cli();
		pKCSBuf = &m_KCSBuf [KCS_1_CHANNEL];
		pKCSBuf->KCSRcvPktIx = 0;
		pKCSBuf->KCSSendPktIx = 0;
		pKCSBuf->KCSSendPktLen = 0;
		pKCSBuf->KCSPhase = KCS_PHASE_IDLE;
		pKCSBuf->KcsINuse = 1;
		filp->f_op = &kcs1_fops;
		//restore_flags(flags);
		break;
	
	case 2:
		dbgprint(KERN_ALERT "kcs_open, case is 2\n");
		//save_flags(flags); cli();
		pKCSBuf = &m_KCSBuf [KCS_2_CHANNEL];
		pKCSBuf->KCSRcvPktIx = 0;
		pKCSBuf->KCSSendPktIx = 0;
		pKCSBuf->KCSSendPktLen = 0;
		pKCSBuf->KCSPhase = KCS_PHASE_IDLE;
		pKCSBuf->KcsINuse = 1;
		filp->f_op = &kcs2_fops;
		//restore_flags(flags);
		break	;
	case 3:
		pKCSBuf = &m_KCSBuf [BBSP_CHANNEL];
                pKCSBuf->KCSRcvPktIx = 0;
                pKCSBuf->KCSSendPktIx = 0;
                pKCSBuf->KcsINuse = 1;
                filp->f_op = &kcs3_fops;
		break;
#ifdef SUPPORT_PORT80
	case 4:
		pKCSBuf = &m_KCSBuf [PORT80_CHANNEL];
				pKCSBuf->KCSRcvPktIx = 0;
				pKCSBuf->KCSSendPktIx = 0;
				pKCSBuf->KcsINuse = 1;
				filp->f_op = &kcs4_fops;
		break;
#endif

	default:
		return -ENXIO;
        }
        return 0;
}



static ssize_t kcs1_read(struct file *filp, char * buf,size_t count,loff_t *ppos)
{
	/* This call will sleep and forms a complete IPMI Packet on Return */
	return ( KcsRequest( (void *)buf , KCS_1_CHANNEL));
}



static ssize_t kcs2_read(struct file *filp, char * buf,size_t count,loff_t *ppos)
{
	/* This call will sleep and forms a complete IPMI Packet on Return */
	return ( KcsRequest((void *)buf , KCS_2_CHANNEL));
}


void 
Kcs_OsInitSleepStruct(Kcs_OsSleepStruct *Sleep)
{
	init_waitqueue_head(&(Sleep->queue));
	init_timer(&(Sleep->tim));
	Sleep->Timeout = 0;
	return;
}

void 
Kcs_OsWakeupOnTimeout(Kcs_OsSleepStruct *Sleep)
{
	int dtimer;

	/* Wakeup Process and Kill timeout handler */
	dtimer=del_timer(&(Sleep->tim));
	wake_up(&(Sleep->queue));

	if (Sleep->Timeout)
		printk(KERN_ERR "WARNING:Wakeup for Sleep Struct Called after timeout\n");
	return;
}



static ssize_t kcs1_write (struct file *file, const char *buf, size_t count,
                             loff_t *offset)
{
//	printk("kcs1_write: Entering  \n");

	return ( KcsWriteRequest( (void *)buf , count ,KCS_1_CHANNEL));


}

static ssize_t kcs2_write (struct file *file, const char *buf, size_t count,
                             loff_t *offset)
{

	//printk("kcs2_write: Entering  \n");

	return ( KcsWriteRequest((void *)buf , count ,KCS_2_CHANNEL));


}


/* bbsp write call  */
static ssize_t bbsp_write (struct file *file, const char *buf, size_t count, 
				loff_t *offset)
{
	bbsp_writereg(buf[0]);	
	return sizeof(buf[0]);
}



/* bbsp_read call*/
static ssize_t bbsp_read(struct file *filp, char *buf, size_t count, loff_t *ppos)
{
        buf [0] = bbsp_readreg();
	return sizeof(buf[0]);
}


/* bbsp write register - core function  */
static int bbsp_writereg( char val)
{
	down(&g_sem);
	KCS_DATA_OUT_REG(BBSP_CHANNEL,val);
	SET_OBF_STATUS(BBSP_CHANNEL);
	up(&g_sem);
	return 0;
}

/* bbsp read register - core function */
static char bbsp_readreg()
{
	KCSBuf_T*               pKCSBuf;
	pKCSBuf = &m_KCSBuf [BBSP_CHANNEL];
        return pKCSBuf->pKCSRcvPkt[0];
}
#ifdef SUPPORT_PORT80
static ssize_t port80_write (struct file *file, const char *buf, size_t count, 
				loff_t *offset)
{
	port80_writereg(buf[0]);	
	return sizeof(buf[0]);
}

/* port80_read call*/
static ssize_t port80_read(struct file *filp, char *buf, size_t count, loff_t *ppos)
{
        buf [0] = port80_readreg();
	return sizeof(buf[0]);
}
/* port80 write register - core function  */
static int port80_writereg( char val)
{
	down(&g_sem);
	KCS_DATA_OUT_REG(PORT80_CHANNEL,val);
	SET_OBF_STATUS(PORT80_CHANNEL);
	up(&g_sem);
	return 0;
}

/* port80 read register - core function */
static char port80_readreg()
{

	KCSBuf_T*               pKCSBuf;
	pKCSBuf = &m_KCSBuf [PORT80_CHANNEL];
    return pKCSBuf->pKCSRcvPkt[0];
}

#endif
static size_t KcsWriteRequest (const char *buf, size_t count, u8 ChannelNum)
{

	//int i;
	IPMICmdMsg_T*	pKCSCmdMsg;	
	KCSBuf_T*	    pKCSBuf;	
	
	switch (ChannelNum) {
		
		case 0: pKCSCmdMsg = &m_KCS1CmdMsg; 
				pKCSBuf    = &m_KCSBuf [KCS_1_CHANNEL];
				break;
		
		case 1: pKCSCmdMsg = &m_KCS2CmdMsg; 
				pKCSBuf    = &m_KCSBuf [KCS_2_CHANNEL];
				break;
        
		case 2: pKCSCmdMsg = &m_KCS3CmdMsg; 
				pKCSBuf    = &m_KCSBuf [KCS_3_CHANNEL];
				break;
#ifdef SUPPORT_PORT80
		case 3: pKCSCmdMsg = &m_KCS4CmdMsg; 
				pKCSBuf    = &m_KCSBuf [KCS_4_CHANNEL];
				break;
#endif
		default: 
				printk(KERN_ERR "Error in KcsWriteRequest \n");	   
				return -ENXIO;

	}


//	printk("KcsWriteRequest:Entering\n");

	/* Copy from user data area to kernel data area*/
	__copy_from_user((void *)(pKCSCmdMsg->Response),(void *)buf,count);
    
	pKCSCmdMsg->ResponseSize = count;

	/* Send the response to KCS Channel */
	SendKCSResponse (ChannelNum, pKCSCmdMsg);

	/* Initialize Sleep Structure for the First Time */
	if (pKCSBuf->KcsResFirstTime)
	{
		Kcs_OsInitSleepStruct(&(pKCSBuf->KcsResWait));
		pKCSBuf->KcsResFirstTime = 0;
	}
	
	pKCSBuf->KcsWtIFActive = 1;
	pKCSBuf->TxReady = 0;
	pKCSBuf->KcsWtIFActive = 0;
	return count;
}

static void KCSRcvByte(u8 ChannelNum)
{

	u8		        Status;
	u8	    	    DummyByte;
	KCSBuf_T*		pKCSBuf;

	Status = 0;	
	//printk("1");



	pKCSBuf = &m_KCSBuf [ChannelNum];
	
	/* Read the Present Status of KCS Port */
	READ_KCS_STATUS_REG (ChannelNum, Status);
	//printk ("KCS Status = 0x%x\n", Status);

#ifdef  SUPPORT_BBSP	
	/* if its channel belongs to BBSP then its BBSP. blindly read the data reg*/
	if ( ChannelNum == BBSP_CHANNEL )
	{
		KCS_DATA_IN_REG(BBSP_CHANNEL,pKCSBuf->pKCSRcvPkt[0]);
		SET_KCS_STATE( ChannelNum, KCS_WRITE_STATE);
		SET_OBF_STATUS(BBSP_CHANNEL);
		is_new_bbspdata =1;
		return;	
	}
#endif	
#ifdef SUPPORT_PORT80
	if ( ChannelNum == PORT80_CHANNEL )
	{
		KCS_DATA_IN_REG(PORT80_CHANNEL,pKCSBuf->pKCSRcvPkt[0]);
		SET_KCS_STATE( PORT80_CHANNEL, KCS_WRITE_STATE);
		SET_OBF_STATUS(PORT80_CHANNEL);
		return;	
	}
#endif
	/* If write to command register */
	if (IS_WRITE_TO_CMD_REG (Status))
	{
		

        u8	Cmd;
		
		Cmd = 0;
		
		//printk ("WriteToCmdReg\n");
		/* Set the status to WRITE_STATE */
		SET_KCS_STATE (ChannelNum, KCS_WRITE_STATE);
		/* Read the command */
		KCS_CMD_REG (ChannelNum, Cmd);
		
	//	printk ("CMD = 0x%x\n",Cmd);
		switch (Cmd)
		{
		case KCS_WRITE_START :

            		//printk ("KCSWriteStart\n");
			/* Set the Index to 0 */
			pKCSBuf->KCSRcvPktIx = 0;
			/* Set the phase to WRITE */
			pKCSBuf->KCSPhase = KCS_PHASE_WRITE;

			break;

		case KCS_WRITE_END 	 :

            /* Set the phase to write end */
			//printk ("KCSWriteEnd\n");
			pKCSBuf->KCSPhase = KCS_PHASE_WRITE_END;

			break;

		case KCS_ABORT       : 

            /* Set the error code */
			//printk ("KCSAbort\n");

			if (KCS_NO_ERROR == pKCSBuf->KCSError)
			{
				pKCSBuf->KCSError = KCS_ABORTED_BY_COMMAND;
			}
			/* Set the phase to write end */
			pKCSBuf->KCSPhase = KCS_PHASE_ABORT;
			/* Set the abort phase to be error1 */
			pKCSBuf->AbortPhase = ABORT_PHASE_ERROR1;
			
			/* Send the dummy byte	*/
			KCS_DATA_OUT_REG (ChannelNum, 0);
			//SA Set OBF Byte
			SET_OBF_STATUS(ChannelNum);

			break;

		default :

			/* Set the error code */
			pKCSBuf->KCSError = KCS_ILLEGAL_CONTROL_CODE;
			/* Invalid command code - Set an error state */
			SET_KCS_STATE (ChannelNum, KCS_ERROR_STATE);
			/* Set the phase to error phase */
			pKCSBuf->KCSPhase = KCS_PHASE_ERROR;
			break;
		}
	}
	else
	{
		int i;

		switch (pKCSBuf->KCSPhase)
		{
		case KCS_PHASE_WRITE :

            		/* Set the state to write state */
			SET_KCS_STATE (ChannelNum, KCS_WRITE_STATE);
			/* Read the BYTE from the data register */
			KCS_DATA_IN_REG (ChannelNum,pKCSBuf->pKCSRcvPkt [pKCSBuf->KCSRcvPktIx]);

			if (pKCSBuf->KCSRcvPktIx < MAX_KCS_PKT_LEN)
			{
				pKCSBuf->KCSRcvPktIx++;
			}
			break;
			
        case KCS_PHASE_WRITE_END :

            /* Set the state to READ_STATE */
			SET_KCS_STATE (ChannelNum, KCS_READ_STATE);

			/* Read the BYTE from the data register */
			KCS_DATA_IN_REG (ChannelNum, pKCSBuf->pKCSRcvPkt [pKCSBuf->KCSRcvPktIx]);

			pKCSBuf->KCSRcvPktIx++;

			//SA lets print all data received from SMS

			//printk("Received data block \n");
			dbgprint("Total bytes received 0x%x\n", pKCSBuf->KCSRcvPktIx );
			for(i=0;i < pKCSBuf->KCSRcvPktIx ; i++)
			{
				dbgprint("0x%x ", pKCSBuf->pKCSRcvPkt [i]);
			}
			dbgprint("\npKCSBuf->KcsIFActive:%x\n",pKCSBuf->KcsIFActive);


			/* Signal receive data ready */
			/* Move to READ Phase */
			pKCSBuf->KCSPhase = KCS_PHASE_READ;

			
			/* SA Wakeup KcsRequest to notify Request Packet is ready*/
			/* Wakeup only if KCS device is opened and is being used */
			if ( pKCSBuf->KcsINuse)
				pKCSBuf->KcsWakeup = 1;
			if (pKCSBuf->KcsIFActive)
			{
				dbgprint("Request received successfully\n");
				Kcs_OsWakeupOnTimeout(&(pKCSBuf->KcsReqWait));
			}
                //else some error...
			break;

		case KCS_PHASE_READ		 :

            /* If we have reached the end of the packet move to idle state */
			if (pKCSBuf->KCSSendPktIx == pKCSBuf->KCSSendPktLen)
			{
				SET_KCS_STATE (ChannelNum, KCS_IDLE_STATE);
			}
			/* Read the byte returned by the SMS */
			{ 
				u8 b = 0;
				KCS_DATA_IN_REG (ChannelNum, b); 
				//SA Need to clear IBF
				//sa_0111 CLEAR_IBF_STATUS(ChannelNum);

				if (b != KCS_READ)
				{
					//printk("Set Error State \n");
					SET_KCS_STATE (ChannelNum, KCS_ERROR_STATE);
					KCS_DATA_OUT_REG (ChannelNum, 0);
					//SA Set OBF Byte
					SET_OBF_STATUS(ChannelNum);
					break;
				}
			}
			/* If we are finished transmitting, send the dummy byte */
			if (pKCSBuf->KCSSendPktIx == pKCSBuf->KCSSendPktLen) 
			{
				pKCSBuf->KCSPhase = KCS_PHASE_IDLE;
				KCS_DATA_OUT_REG (ChannelNum, 0);
				//SA Set OBF Byte
				SET_OBF_STATUS(ChannelNum);

				/* Set Transmission Complete */
				pKCSBuf->TxReady		= 1;
				/* Wakeup Sleeping Process */
				//if(pKCSBuf->KcsWtIFActive)
				//	Kcs_OsWakeupOnTimeout(&(pKCSBuf->KcsResWait));
                break;
			}
			/* Transmit the next byte from the send buffer */
			KCS_DATA_OUT_REG (ChannelNum, pKCSBuf->pKCSSendPkt [pKCSBuf->KCSSendPktIx]);
			//SA Set OBF Byte
			SET_OBF_STATUS(ChannelNum);
			pKCSBuf->KCSSendPktIx++;
			break;
			
		case KCS_PHASE_ABORT 	 :

            switch (pKCSBuf->AbortPhase) 
			{
			case ABORT_PHASE_ERROR1 :
				/* Set the KCS State to READ_STATE */
				SET_KCS_STATE (ChannelNum, KCS_READ_STATE);
				/* Read the Dummy byte	*/
				KCS_DATA_IN_REG (ChannelNum, DummyByte); 
				//SA Need to clear IBF
				//sa_0111 CLEAR_IBF_STATUS(ChannelNum);
				/* Write the error code to Data out register */
				KCS_DATA_OUT_REG (ChannelNum, pKCSBuf->KCSError);
				//SA Set OBF Byte
				SET_OBF_STATUS(ChannelNum);
				/* Set the abort phase to be error2 */
				pKCSBuf->AbortPhase = ABORT_PHASE_ERROR2;

				break;
				
			case ABORT_PHASE_ERROR2 :

                /**
				 * The system software has read the error code. Go to idle
				 * state.
				**/
				SET_KCS_STATE (ChannelNum, KCS_IDLE_STATE);

				/* Read the Dummy byte	*/
				KCS_DATA_IN_REG (ChannelNum, DummyByte); 

				pKCSBuf->KCSPhase = KCS_PHASE_IDLE;
				pKCSBuf->AbortPhase = 0;

				/* Send the dummy byte	*/
				KCS_DATA_OUT_REG (ChannelNum, 0);
				//SA Set OBF Byte
				SET_OBF_STATUS(ChannelNum);

			}
			break;

			default:
				/* Read the Dummy byte	*/
				KCS_DATA_IN_REG (ChannelNum, DummyByte); 
				///printk(KERN_ALERT "Set Error State \n");
				SET_KCS_STATE (ChannelNum, KCS_ERROR_STATE);
				KCS_DATA_OUT_REG (ChannelNum, 0);
				//SA Set OBF Byte
				SET_OBF_STATUS(ChannelNum);
		}
	}



}


/**
 * @brief Prepare KCS buffer and send it on given channel.
 * @param ChannelNum - KCS channel number.
 * @param MsgPkt     - Response message packet.
**/
static void
SendKCSResponse (u8 ChannelNum, IPMICmdMsg_T *pKCSCmdMsg)
{
	KCSBuf_T*	pKCSBuf = &m_KCSBuf [ChannelNum];

	//printk("SendKCSResponse() Entering: Channel no = %x\n", ChannelNum);

	/* Update the send buffer and associated indexes */
	pKCSBuf->pKCSSendPkt	= pKCSCmdMsg->Response;
	pKCSBuf->KCSSendPktLen	= pKCSCmdMsg->ResponseSize;
	pKCSBuf->KCSSendPktIx   = 0;

	/* Send the first byte */
	pKCSBuf->KCSSendPktIx++;
	//printk("First byte = 0x%x",pKCSBuf->pKCSSendPkt[0]);
	KCS_DATA_OUT_REG (ChannelNum, pKCSBuf->pKCSSendPkt[0]);
	//SA Set OBF Byte
	SET_OBF_STATUS(ChannelNum);

	/* From now onwords the data is sent from the IBF Interrupt handler */
	return;
}

long 
Kcs_OsSleepOnTimeout(Kcs_OsSleepStruct *Sleep,u8 *Var,long msecs)
{
	long timeout;	/* In jiffies */
	int  dtimer;
	u8 *volatile Condition = Var;

	/* Convert milliseconds into jiffies*/
	timeout = (HZ*msecs)/1000;

	/* Start the timeout */
	Sleep->tim.function = SleepTimeOut;
	Sleep->tim.data     = (unsigned long)Sleep;
	Sleep->tim.expires  = jiffies+timeout;
	Sleep->Timeout      = 0;
	if (msecs)
		add_timer(&(Sleep->tim));

//#if DEBUG_ARCH
	//printk("Sleeping on Sleep Structure\n");
//#endif

	/* Sleep on the Condition for a wakeup */
	wait_event_interruptible(Sleep->queue,((*Condition)||(Sleep->Timeout)));

	/* Normally del_timer will be called by Wakeup or timeout routine */
	/* But when wakeup is called before add_timer() and wait_event(),
      it will not be killed by wakeup routine and will be killed only
	   after a timeout happens. So we kill it here as soon as possible */
	if (msecs)
		dtimer = del_timer(&(Sleep->tim));

	if (Sleep->Timeout)
		return 0;
	return timeout;
}

static
void
SleepTimeOut(unsigned long SleepPtr)
{	
	Kcs_OsSleepStruct *Sleep;

	/* Set Timeout and Wakeup Sleep Process */
	Sleep = (Kcs_OsSleepStruct *)SleepPtr;
	Sleep->Timeout = 1;
	wake_up(&(Sleep->queue));

	return;
}

static int kcs_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg)
{
	u8 Status = 0;
	//u8 kcsifcnum;
	kcs_data_t KCSData;

	if ( copy_from_user(&KCSData, (void*)arg, sizeof(kcs_data_t)) )
		return -EFAULT;

	//printk ("Inside kcs_ioctl\n");


	switch(cmd)
	{
	case SET_SMS_BIT:
		/* Channel 1 (KCS4 interface on Pilot-II) is used for SMS */
		READ_KCS_STATUS_REG (KCSData.kcsifcnum, Status);
		/* We should check for IBF bit before clearing it by writing 1 back */
		if (IBF_BIT_SET(Status))
			KCSRcvByte(1);
		Status = Status | 0x04;
		WRITE_KCS_STATUS_REG (KCSData.kcsifcnum, Status);
		break;
	case CLEAR_SMS_BIT:
		/* Channel 1 (KCS4 interface on Pilot-II) is used for SMS */
		READ_KCS_STATUS_REG (KCSData.kcsifcnum, Status);
		/* We should check for IBF bit before clearing it by writing 1 back */
		if (IBF_BIT_SET(Status))
			KCSRcvByte(1);
		Status = Status & ~0x04;
		WRITE_KCS_STATUS_REG (KCSData.kcsifcnum, Status);
		break;
    case GET_LPCRST_STATUS:
        // read the LPC Reset flag
        copy_to_user ((void*)arg, (void*)& sg_u8LPCReset, sizeof(sg_u8LPCReset));
        break;
    case CLEAR_LPCRST_STATUS:
        // reset the LPC Reset flag
        sg_u8LPCReset = 0;
        break;
#ifdef SUPPORT_BBSP
    case BBSP_STATUS:
		READ_KCS_STATUS_REG(BBSP_CHANNEL,Status);
		/*Check whether OBF is cleared*/	
		if ( Status & 1 )	
		{
			KCSData.is_host_read = 1;	
			SET_OBF_STATUS(BBSP_CHANNEL);
		}
		else
			KCSData.is_host_read = 0;
		/* write the data sent by the IPMI*/	
		bbsp_writereg(KCSData.bbsp_data);
		
		/* Did we received any new data from host */	
		if ( is_new_bbspdata )
		{
			KCSData.is_data_new = 1;
			KCSData.bbsp_data = bbsp_readreg();
			is_new_bbspdata = 0;
		}	
		else
			KCSData.is_data_new = 0;
        	if ( copy_to_user((void*)arg,(void*)&KCSData, sizeof(kcs_data_t)) )
			return -EFAULT;
	
		break;	
#endif
	default:
		//printk(KERN_ALERT "Invalid IOCTL command\n");
		break;
	}

	return 0;
}

static int kcs1_release(struct inode * inode, struct file * filp)
{
	KCSBuf_T*		pKCSBuf;
	
	pKCSBuf = &m_KCSBuf [KCS_1_CHANNEL];
	pKCSBuf->KcsINuse = 0;
	dbgprint("Channel %x Closed\n", KCS_1_CHANNEL);
	return 0;
}

static int kcs2_release(struct inode * inode, struct file * filp)
{
	KCSBuf_T*		pKCSBuf;
	
	pKCSBuf = &m_KCSBuf [KCS_2_CHANNEL];
	pKCSBuf->KcsINuse = 0;
	
	dbgprint("Channel %x Closed\n", KCS_2_CHANNEL);
	return 0;
}

static int bbsp_release(struct inode * inode, struct file * filp)
{
	KCSBuf_T*	pKCSBuf;
	
	pKCSBuf = &m_KCSBuf [ KCS_3_CHANNEL];
	pKCSBuf->KcsINuse = 0;
	
	dbgprint ( "BBSP channel closed\n");
	return 0;
}
#ifdef SUPPORT_PORT80
static int port80_release(struct inode * inode, struct file * filp)
{
	KCSBuf_T*	pKCSBuf;
	
	pKCSBuf = &m_KCSBuf [ KCS_4_CHANNEL];
	pKCSBuf->KcsINuse = 0;
	
	dbgprint (KERN_ALERT "port80 channel closed\n");
	return 0;
}

#endif
/* Function for  Register the Resetfunction in KCS function pointer  */

int
RegisterforHostReset(int (*pt2Func)(void))
{
	if (ResetFunction != NULL)
		return 1;
	ResetFunction=pt2Func;
	return 0;
}

/*  Unregister the Reset function in KCS funciton  pointer */
void 
UnregisterforHostReset(void)
{
	ResetFunction=NULL;
	return;
}

/*--------------------------- Module Information Follows --------------------------*/
module_init (init_kcs_module);
module_exit (cleanup_kcs_module);

MODULE_AUTHOR(" Connie Yin - American Megatrends Inc");
MODULE_DESCRIPTION("This is LPC driver module.");
MODULE_LICENSE("GPL");

EXPORT_SYMBOL(RegisterforHostReset);
EXPORT_SYMBOL(UnregisterforHostReset);

#endif	/* MODULE */

