/*************************************************************
Module		: POS Register driver module
*************************************************************/
/* Version Number of this module */
#define SAMPLE_MAJOR	1
#define SAMPLE_MINOR	0


#include <linux/module.h>
#include <linux/kernel.h>	/* printk() 		*/
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/ide.h>

#define	dbgprint	if(Debug_Enable) printk

#include "pos.h"

#define POS_MAJOR	253
#define POS_DEV_NAME   "POS"

static ssize_t pos_read(struct file *filp, char * buf,size_t count,loff_t *ppos);
static ssize_t pos_write (struct file *file, const char *buf, size_t count,loff_t *offset);
static int pos_open(struct inode * inode, struct file * filp);



static struct file_operations pos_fops = {
	owner:		THIS_MODULE,
	read:		pos_read,
	write:		pos_write,
	ioctl:		NULL,
	open:		pos_open,
	release:	NULL,
};

/** sysfs  data structures **/
extern struct class * get_hwmon_class(void);
static ssize_t showposreg(struct class *,  char *buffer);
static ssize_t storeposreg(struct class *,  const char *buffer, size_t count);

/** read write call prototypes **/
static unsigned long read_reg( void );
static void write_reg(unsigned char buf);
static CLASS_ATTR(pos,0666,showposreg,storeposreg);
struct class * cls = NULL;
/* semaphore to provide mutex */
struct semaphore g_sem;

/*show posreg*/
static ssize_t showposreg(struct class *cls,  char *buffer)
{
	return	sprintf( buffer, "%lx\n", *(POS_REG));	
}

/*store posreg*/
static ssize_t storeposreg(struct class *kobj,  const char *buffer, size_t count)
{
	unsigned long val = 0;
	sscanf ( buffer, "%lx", &val);
	write_reg(val);
	return count;
}

/** syfs ends **/


/* read reg */
static unsigned long read_reg()
{
	return *(POS_REG);
}

/* write reg */
static void write_reg( unsigned char data)
{
	down(&g_sem);
	*(POS_REG) = data;
	up(&g_sem);
	return;
}
static int
init_pos_module(void)
{
	int 	err;

	printk(KERN_ALERT "POS  Driver Version %d.%d\n",SAMPLE_MAJOR,SAMPLE_MINOR);
	printk(KERN_ALERT "Copyright (c) 2007 American Megatrends Inc.\n");

	/* ----  Register the character device ------------------- */
	if ((err = register_chrdev(POS_MAJOR, POS_DEV_NAME, &pos_fops)) < 0)
	{
		printk (KERN_ERR "failed to register device (err %d)\n", err);
		return -ENODEV;
	}
	/* Initialize semaphore */
	init_MUTEX(&g_sem);
	
	cls = get_hwmon_class();

	if( cls )
	{
		err = class_create_file(cls, &class_attr_pos);	
		if (err)
		{
			printk (KERN_ALERT "POS: failed to create hwmon class file");
		}	
	}
			
	return 0;
}

static void
cleanup_pos_module(void)
{
        printk(KERN_ALERT "Unloading POS Module ..............\n");


        /* unregister the device */
        unregister_chrdev(POS_MAJOR, "POS");

        /*sysfs cleanup*/
        if ( cls)
                class_remove_file(cls, &class_attr_pos);

        printk(KERN_ALERT "POS module unloaded sucessfully.\n");

}
   


static int pos_open(struct inode * inode, struct file * filp)
{

        if(filp->f_flags & 010000000)
        {
                return 0;
        }
        filp->f_op =&pos_fops;
	return  0;
}

static ssize_t pos_read(struct file *filp, char * buf,size_t count,loff_t *ppos)
{
	if ( (buf != NULL) && (count == 1) )
	{
		*buf = read_reg();
		return sizeof(unsigned char);
	} 
	else
	{
		printk(KERN_ERR "Pos-read failed.\n");
	}	
	return 0; //to do		
}

static ssize_t pos_write (struct file *file, const char *buf, size_t count,loff_t *offset)
{
	if ( (buf != NULL) && (count <= 1))
	{
		write_reg(*buf);
		return sizeof(unsigned char);
	}
	else
	{
		printk(KERN_ERR "Pos-write failed.\n");
	}
	return 0; //to do
}



/*--------------------------- Module Information Follows --------------------------*/
module_init (init_pos_module);
module_exit (cleanup_pos_module);

MODULE_AUTHOR("American Megatrends Inc");
MODULE_DESCRIPTION("This is POS driver module.");
MODULE_LICENSE("GPL");

