/*-------------------------------------------------------------------------
	Copyright (C) 2008-2009 American Megatrends Inc.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-------------------------------------------------------------------------*/
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/wait.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>
#include "i2c-data.h"
#include "i2c-hardware.h"
#include "gpio_vsc.h"
#include "gpio_info.h"


#define I2C_ID			1
#define IRQ_I2C			7

#define I2C_NODE_0		0
#define I2C_NODE_1		1
#define I2C_NODE_2		2
#define I2C_NODE_3		3
#define I2C_NODE_4		4
#define I2C_NODE_5		5

#define I2C_REMAP_FAILS			0x0A
#define I2C_MEM_CHECK_REG_FAILS	0xBB
#define I2C_REQ_MEM_REG_FAILS	0xCC
#define I2C_REGIS_IRQ_FAILS		0xDD
#define SET_IIC_OPTION			2


int i2c_node[BUS_COUNT] =
{
#if (BUS_COUNT >= 1)
	I2C_NODE_0, 
#endif
#if (BUS_COUNT >= 2)
	I2C_NODE_1, 
#endif
#if (BUS_COUNT >= 3)
	I2C_NODE_2, 
#endif
#if (BUS_COUNT >= 4)
	I2C_NODE_3, 
#endif
#if (BUS_COUNT >= 5)
	I2C_NODE_4, 
#endif
#if (BUS_COUNT >= 6)
	I2C_NODE_5
#endif
};

static struct i2c_adapter vsc_ops[BUS_COUNT];
extern struct i2c_vsc_data vsc_data_ptr[BUS_COUNT];
extern int write_init_st(int option,int value);

void *i2c_remap_addr;
static int bus_count = BUS_COUNT;
static short init_iic_status=0;
extern int vsc_get_recovery_info(int bus, bus_recovery_info_T* info);
extern int vsc_set_recovery_info(int bus, bus_recovery_info_T* info);

gpio_dev_t *g_gpio_dev = NULL;
extern void * get_gpio_dev(void);

static int __init i2c_vsc_init(void)
{
	int i=0,j=0;
	int err, ret=0;
	uint32_t i2c_base_addr;

	printk(KERN_ALERT "Loading VSC452 I2C driver .. 2010-08-27 *************\n");

	g_gpio_dev = get_gpio_dev();

	/* Remap registers */
	i2c_base_addr = I2C_BASE;
	if ((i2c_remap_addr = ioremap_nocache(i2c_base_addr, I2C_REG_SIZE)) == NULL) {
		printk(KERN_ERR "Failed to map I2C IO space %08x-%08x to memory", i2c_base_addr, i2c_base_addr + I2C_REG_SIZE - 1);
		init_iic_status=I2C_REMAP_FAILS;
		ret = -EIO;
		goto fail;
	}

	/* Check I2C memory region */
	if ((err = check_mem_region(i2c_base_addr, I2C_REG_SIZE)) < 0) {
		printk(KERN_ERR "I2C IO space %08x-%08x already in use(err %d)", i2c_base_addr, i2c_base_addr + I2C_REG_SIZE - 1, err);
		init_iic_status=I2C_MEM_CHECK_REG_FAILS;
		ret = -EBUSY;
		goto fail;
	}

	/* Request for the memory region */	
	if (request_mem_region(i2c_base_addr, I2C_REG_SIZE, DRIVER_NAME) == NULL) { 
		printk(KERN_ERR "IO space %08x-%08x i2c request mem region failed (err %d)", i2c_base_addr, i2c_base_addr + I2C_REG_SIZE - 1, err);
		init_iic_status=I2C_REQ_MEM_REG_FAILS;
		ret = -ENODEV;
		goto fail;
	}

	/* Initialize driver's itnernal data structure */
	for (i=0; i<bus_count; i++ ) {
		i2c_init_internal_data(i);
	}

	/* 
	 * Registering I2C IRQ for all available bus
	 * I2C driver uses Shared IRQ (IRQ Number 7),shared by all available bus   
 	 */ 	 
	for (i=0; i<bus_count; i++) {
		/*i2c_vsc_disable_all_interrupts(i);*/
		if (request_irq(IRQ_I2C, i2c_handler, IRQF_SHARED, "i2c_vsc", &i2c_node[i] ) < 0 ) {
			printk(KERN_ERR "Request for irq %d failed\n", IRQ_I2C);
			init_iic_status=I2C_REGIS_IRQ_FAILS;
			ret = -ENODEV;
			goto fail;
		}
	}
	
	/* Initialize all I2C bus
	 * By Default all the I2C bus are initialised as Slave ,waiting for the interrupts across the physical hardware bus 
	 */ 
	for (i=0; i<bus_count; i++) {
		i2c_init_hardware(i);
	}

	/* Register ourselves with the upper layers */
	for (i=0; i<bus_count; i++) {
		/* Init bus recovery counter (used for keeping count of bus recovery attempts - informational only) */
		vsc_data_ptr[i].bus_recovery_cnt = 0;
		/* Init bus busy counter */
		vsc_data_ptr[i].bus_busy_cnt = 0;
		/* Init device failure counter (tracks number of failures per device */
		for (j=0; j<128; j++)
			vsc_data_ptr[i].dev_fail_cnt[j] = 0;

		/* Set up i2c_adapter structs */
		vsc_ops[i].owner = THIS_MODULE;
		vsc_ops[i].class = I2C_CLASS_HWMON;
		vsc_ops[i].id = I2C_ID;
		vsc_ops[i].algo_data = &vsc_data_ptr[i];
		vsc_ops[i].nr = i;
		sprintf(vsc_ops[i].name, "i2c-vsc452-%d", i);

		/* Add the bus via the algorithm code */
		if (i2c_vsc_add_bus(&vsc_ops[i]) != 0) {
			printk(KERN_ERR "Cannot add bus %d to algorithm layer\n", i);
			return (-ENODEV);
		}
	}
	write_init_st(SET_IIC_OPTION,init_iic_status);
	printk(KERN_ALERT "VSC452 I2C sucessfully loaded\n");
	return ret;
	
fail:
	i2c_vsc_exit();
	return ret;
}

void i2c_vsc_exit(void)
{
	int i;
	uint32_t i2c_base_addr;

	printk (KERN_ALERT "Unloading VSC452 I2C driver ..\n");
	
	if(init_iic_status==0)
	{
		init_iic_status=0xff;
	}
	write_init_st(SET_IIC_OPTION,0xff);
	/* Remove the bus via algorithm code */
	for (i=0; i<bus_count; i++) {
		i2c_vsc_del_bus(&vsc_ops[i]);
	}

	i2c_base_addr = I2C_BASE;

	/* Free IRQ before unload */
	for (i=0; i<bus_count; i++) {
		/* i2c_vsc_disable_all_interrupts(i); */
		free_irq(IRQ_I2C,&i2c_node[i]);
	}
	
	/* Release the memory occupied */  
	release_mem_region(i2c_base_addr,I2C_REG_SIZE);

	/* Unmap registers */
	iounmap(i2c_remap_addr); 

	printk(KERN_ALERT "VSC452 I2C driver Unloaded Successfully\n");
	return;
}

/* Return bitfield describing our capabilities */
static u32 vsc_func(struct i2c_adapter *adap)
{
	return (I2C_FUNC_I2C);
}


/* Algorithm level handling of ioctls and other control commands */
static int vsc_control(struct i2c_adapter *i2c_adap, 
			unsigned long cmd, unsigned long arg)
{
	int retval = 0;
	bus_recovery_info_T  bus_recovery_info;
	switch (cmd) {
#ifdef I2C_EXTENDED_CMDS
	case I2C_SLAVEREAD:
		/* Unsupported (for now) */
		retval = -ENOTTY;
		break;

	case I2C_SLAVEWRITE:
		/* Unsupported (for now) */
		retval = -ENOTTY;
		break;

	case I2C_SET_HOST_ADDR:
		/* i2c_vsc_set_slave( i2c_adap->nr, arg ); */
		/* Unsupported (for now) */
		retval = -ENOTTY;
		break;

	case I2C_GET_HOST_ADDR:
            	/* retval = i2c_vsc_get_slave( i2c_adap->nr ); */
		/* Unsupported (for now) */
		retval = -ENOTTY;
		break;

	case I2C_RESET:
		printk(KERN_ALERT "I2C%d: Resetting the controller\n", i2c_adap->nr);
		/* Reset the controller */
		i2c_init_internal_data(i2c_adap->nr);
		i2c_init_hardware(i2c_adap->nr);
		break;

#endif /* I2C_EXTENDED_CMDS */
	case I2C_SET_REC_INFO:
		if (copy_from_user(&bus_recovery_info, (void*)arg, sizeof(bus_recovery_info_T)))
			return -EFAULT;

		if (0 != vsc_set_recovery_info(i2c_adap->nr, &bus_recovery_info)) {
			dev_err(&i2c_adap->dev, "vsc_control: set recovery info vsc\n");
		}
		break;

	case I2C_GET_REC_INFO:
		if (0 != vsc_get_recovery_info(i2c_adap->nr, &bus_recovery_info)) {
			dev_err(&i2c_adap->dev, "vsc_control: get recovery info vsc\n");
		}

		if (copy_to_user((void*)arg, &bus_recovery_info, sizeof(bus_recovery_info_T))) 
			return -EFAULT;

		break;

	default:
		dev_err(&i2c_adap->dev, "algo_control: Unknown command\n");
		retval = -ENOTTY;
	}

	return (retval);
}

static struct i2c_algorithm vsc_algo =
{
	.master_xfer = vsc_master_xfer,
	.functionality = vsc_func,
	.i2c_hw_ioctl = vsc_control,
	.slave_recv = vsc_slave_xfer,
#ifdef I2C_BUS_RECOVERY
	.i2c_recovery = vsc_i2c_bus_recovery,
#else
	.i2c_recovery = NULL,
#endif
};


int 
i2c_vsc_add_bus( struct i2c_adapter *i2c_adap )
{
    /* Register new adapter */
    i2c_adap->algo = &vsc_algo;

    /* Set some sensible defaults */
    i2c_adap->timeout = DEFAULT_TIMEOUT;
    i2c_adap->retries = DEFAULT_RETRIES;

    i2c_add_adapter( i2c_adap );
    return(0);
}

int 
i2c_vsc_del_bus( struct i2c_adapter *i2c_adap )
{
    return( i2c_del_adapter( i2c_adap ) );
}

MODULE_DESCRIPTION("This is VSC452 I2C driver module.");
MODULE_LICENSE("GPL");

module_init( i2c_vsc_init );
module_exit( i2c_vsc_exit );
