#include <linux/module.h>
#include <linux/init.h>
#include <linux/moduleparam.h>

#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/types.h>

#include <linux/in.h>
#include <linux/ip.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/tcp.h>
#include <linux/skbuff.h>
#include <linux/mii.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/notifier.h>
#include <linux/reboot.h>



#include "vsc452mac.h"


#define MODULE

//#define HAVE_MULTICAST


//Base address of nics
#define VSCMAC_BASE0		0xB0400000
#define VSCMAC_BASE1		0xB0480000

static int multicast = 0;
module_param(multicast, int, 0);


/* Hardware interrupt forcing register */
#define HIF_REG 	(volatile unsigned long *)0xB02008f4

/* Interrupt observe register - network related bit masks */
#define IOR0_EMACIS0_BITMASK		0x40000000
#define IOR0_EMACIS1_BITMASK		0x80000000

/* Interrupt observe registers */
#define IOR0_REG 	(volatile unsigned long *)0xB02008f8
#define IOR1_REG 	(volatile unsigned long *)0xB02008fc



#define MAX_ETH_FRAME_SIZE			1536
#define MAX_ETH_UNTAGGED_FRAME_SIZE		1518
#define MIN_ETH_FRAME_SIZE			64
#define STOP_THRESHOLD				0x300				// (6144/2)
#define FIFO_CLR_ON_OVERFLOW 			0
#define EMPTY_THRESHOLD			0x100


#define VSC452_PHY_ADDR		0x1f    /* VNIC PHY address */
#define VSC452_OCR			0xB02008A4  /* OCR register address */
#define MDIOENA			0x00000400  /* MDIO ENABLE bit */



#define MAX_VSC452MAC		2
//port 0,1 base address
unsigned long vsc452mac_io[] =  {VSCMAC_BASE0,VSCMAC_BASE1};

//port 0,1 irqs (basixally both use same irq 3).
unsigned long vsc452mac_irq[] = { 3,3 };//MIPSCPU_INT_BASE+MIPSCPU_INT_ETH0,
											//	MIPSCPU_INT_BASE+MIPSCPU_INT_ETH1};

#define VSC452MAC_DEBUG	1
#define DRV_NAME		"VSC452MAC"


static const char       version[]="VSC452MAC Driver,(Linux Kernel 2.6.X)  - by AMI\n";

/*global varibale for loopback Test */
int enableLoopback=0;
char rx_buffer[ BUFLEN]; //store the data in the vsc452mac_dma_receive


//prototypes.
#ifdef VSC452MAC_USE_DMA
static void vsc452mac_enable_rx_dma(struct net_device *dev);
static void vsc452mac_enable_tx_dma(struct net_device *dev);
static void vsc452mac_disable_rx_dma(struct net_device *dev);
static void vsc452mac_disable_tx_dma(struct net_device *dev);
static uint32 vsc452mac_dma_alloc_buffers(struct net_device *dev);
static uint32 vsc452mac_dma_free_buffers(struct net_device *dev);
static uint32 vsc452mac_dma_ringsetup(struct net_device *dev);
static void vsc452mac_init_dma(struct net_device *dev);

//transmit functions
static int vsc452mac_dma_send_packet(struct sk_buff *skb, struct net_device *dev);
static uint32	vsc452mac_dma_int_tx(struct net_device *dev);

//receive functions
static uint32	vsc452mac_dma_int_rx(struct net_device *dev, struct napi_struct *napi);
static int vsc452mac_reinitbuffers(struct net_device *dev);
static int vsc452mac_dma_receive(struct napi_struct *napi, int budget);


#endif

static void vsc452mac_writephy(struct net_device *dev, uint8 phy_id, uint8 address, uint32 value);
static uint32 vsc452mac_readphy(struct net_device *dev, uint8 phy_id, uint8 address);
static void vsc452mac_shutdown(unsigned int base);





inline
void pioSt32(uint32 addr, uint32 val)
{
	*((volatile uint32*)addr) = val;
}

inline
void pioLd32(uint32 addr, uint32 * valPtr)
{
	*valPtr = *((volatile uint32*)addr);
}

/* reboot notifiers - starts */

static int vsc_notify_ethernet_driver(struct notifier_block *this, unsigned long code, void *unused);
static struct notifier_block vsc_ethernet_notifier =
{
	.notifier_call = vsc_notify_ethernet_driver,
};

static int vsc_notify_ethernet_driver(struct notifier_block *this, unsigned long code, void *unused)
{

	unsigned int base = 0;
	int i =0;

	//printk ("ethernet reboot notifier invoked\n");
	for (i = 0; i < MAX_VSC452MAC; i++)
        {
		base = vsc452mac_io[i];
#ifdef VSC452MAC_USE_DMA
		/* disable rx dma  */
		pioSt32((base+RDCREG),0);
		/* disable tx dma */
		pioSt32((base+TDCREG),0);
		/* disbale mac receive */
#endif
		vsc452mac_shutdown(base);
	}
      	return NOTIFY_DONE;

}

/* reboot notifier - ends  */

#if VSC452MAC_DEBUG > 2
void
print_packet(unsigned char *data,int len)
{
	int i;
	for(i=0;i<len;i++)
	{
		if (!(i % 16))
			printk("\n");
		printk("%02X ",data[i]);
	}
	printk("\n");
	return;
}
#endif


/******************************************************************
 ******************	MAC ADDRESS RELATED FUNCTIONS******************
 ******************************************************************/
extern unsigned long enetaddr[4][6];

void
FindMACAddress(unsigned int unit, char *mac_addr)
{
	/*Passed to us from bootloader*/
	mac_addr[0] = 0x00;
	mac_addr[1] = 0x40;
	mac_addr[2] = 0xD9;
	mac_addr[3] = 0x02;
	mac_addr[4] = 0xB5;
	if ( 0 == unit )
		mac_addr[5] = 0x1D;
	else
		mac_addr[5] = 0x1E;

//	memcpy(mac_addr,enetaddr[unit],6);
	return;
}


static int
put_mac(struct net_device *dev, void *macaddr)
{
	unsigned long val;
	uint32 base = dev->base_addr;
	uint8 * mac_addr = (uint8*)macaddr;

	//printk("vsc452mac_put_mac\n");

	memcpy(dev->dev_addr,mac_addr, dev->addr_len);

	val = ((uint32)mac_addr[0])<<8 | (uint32)mac_addr[1];
	pioSt32(base+REG_SA0,val);
	val = ((uint32)mac_addr[2])<<8 | (uint32)mac_addr[3];
	pioSt32(base+REG_SA1,val);
	val = ((uint32)mac_addr[4])<<8 | (uint32)mac_addr[5];
	pioSt32(base+REG_SA2,val);
	return 0;
}
void
put_mac_frm_devaddr(struct net_device *dev )
{

	unsigned long val;
	uint32 base = dev->base_addr;
	uint8 * mac_addr = dev->dev_addr;

	//printk("vsc452mac_put_mac\n");

	val = ((uint32)mac_addr[0])<<8 | (uint32)mac_addr[1];
	pioSt32(base+REG_SA0,val);
	val = ((uint32)mac_addr[2])<<8 | (uint32)mac_addr[3];
	pioSt32(base+REG_SA1,val);
	val = ((uint32)mac_addr[4])<<8 | (uint32)mac_addr[5];
	pioSt32(base+REG_SA2,val);

}


void
get_mac(unsigned int base, char *mac_addr)
{
	unsigned long val;

	pioLd32(base+REG_SA0,&val);
	mac_addr[0] = (val>>8)&0xff;
	mac_addr[1] = val&0xff;
	pioLd32(base+REG_SA1,&val);
	mac_addr[2] = (val>>8)&0xff;
	mac_addr[3] = val&0xff;
	pioLd32(base+REG_SA2,&val);
	mac_addr[4] = (val>>8)&0xff;
	mac_addr[5] = val&0xff;

}

void
vsc452_print_mac(int base,char *mac_addr)
{
	int i;
	unsigned char local_mac_addr[6];

	get_mac(base, &local_mac_addr[0]);

	printk("ADDR: ");
	for (i = 0; i < 5; i++)
	{
		printk("%2.2x:", local_mac_addr[i] );
	}
	printk("%2.2x \n", local_mac_addr[5] );


}


static void
vsc452mac_reset( struct net_device* dev )
{
	unsigned int	baseAddr = dev->base_addr;

	printk("vsc452mac_reset\n");

	// turn on huge frame enable, full-duplex, CRC enable
	pioSt32 ( baseAddr + REG_MAC2,  MAC2_FD | MAC2_CE | MAC2_ADPE | MAC2_PCE | MAC2_FLC |MAC2_ED);

	//set transmit/receive inter-packet gaps */
	pioSt32(baseAddr+REG_IPGT, 0x15);
	pioSt32(baseAddr+REG_IPGR,0xC12);

	// turn on receive enable
	pioSt32 ( baseAddr + REG_MAC1,MAC1_TFC|MAC1_RFC);//|MAC1_RE );

	// max frame size
	pioSt32 ( baseAddr +REG_MAXF,0x5ee);

	// Set up almost Full to proper value to avoid fifo overflow
	pioSt32(baseAddr + REG_RFAF, STOP_THRESHOLD );
	pioSt32(baseAddr + REG_RFAE, EMPTY_THRESHOLD );
	pioSt32(baseAddr + REG_TFAF, STOP_THRESHOLD );
	pioSt32(baseAddr + REG_TFAE, EMPTY_THRESHOLD );

	return;
}



/*
 . Function: vsc452mac_enable
 . Purpose: let the chip talk to the outside work
 . Method:
 .	1.  Enable the transmitter
 .	2.  Enable the receiver
 .	3.  Enable interrupts
*/
#if 0
static void
vsc452mac_enable( struct net_device *dev )
{
	unsigned int baseAddr = dev->base_addr;
       char mac_addr[6];
	struct vsc452mac_local *lp = netdev_priv(dev);

	printk("vsc452mac_enable\n");

	/* Set the MAC address  and display */
	put_mac(dev, dev->dev_addr);
	get_mac(baseAddr, mac_addr);
       vsc452_print_mac(baseAddr,mac_addr);

	pioSt32(baseAddr + REG_TRCR, TRCR_RMIIROS|TRCR_RCDR|TRCR_TCDR|TRCR_RFR|TRCR_TFR);
	udelay(5000);

	// Clear all of the interrupt status bits
	// pioSt32(baseAddr + REG_IS, 0x0003ffff);

	// Set up almost Full to proper value to avoid fifo overflow
	// pioSt32(baseAddr + REG_RFAF, STOP_THRESHOLD );

	lp->trans_busy = 0;

	// turn on receive enable
	pioSt32 ( baseAddr + REG_MAC1,MAC1_TFC|MAC1_RFC|MAC1_RE );

	//udelay(5000);
#ifndef VSC452MAC_USE_DMA
      // enable incessary interrupts
       pioSt32 ( baseAddr + REG_ISE, ISE_RFAF|ISE_RSVP|ISE_RFOR|ISE_RGP|ISE_RFO);
#else
	// added dma interrupts n the second line
	pioSt32 ( baseAddr + REG_ISE, TPCINTR|RPCINTR|RQOINTR|RBOIINTR|RFOIINTR);
#endif


	//initial MAC Control Register
	pioSt32 ((baseAddr+REG_MACC),0);

	/* Initialize Transmit and Receive Control Register */
       pioSt32(baseAddr+REG_TRCR, TRCR_RMIIROS);

	return ;
}
#endif

/*
 . Function: vsc452mac_shutdown
 . Purpose:  closes down the SMC91xxx chip.
 . Method:
 .	1. zero the interrupt mask
 .	2. clear the enable receive flag
 .	3. clear the enable xmit flags
 .
 . TODO:
 .   (1) maybe utilize power down mode.
 .	Why not yet?  Because while the chip will go into power down mode,
 .	the manual says that it will wake up in response to any I/O requests
 .	in the register space.   Empirical results do not show this working.
*/
static void
vsc452mac_shutdown( unsigned int baseAddr )
{
	//printk("vsc452mac_shutdown\n");

	// Disable All Interrupts
	pioSt32 ( baseAddr + REG_ISE,0);

	// turn off receive enable
	pioSt32 ( baseAddr + REG_MAC1,0 );

	return;
}

void
vsc452mac_destructor(struct net_device *dev)
{
	//printk("vsc452mac_desctructor\n");
	return;
}



/******************************************************************
 ******************	MAIN HARDWARE PROBE FUNCTION ******************
 ******************************************************************/

/*-----------------------------------------------------------------
 . Function: do_vsc452mac_probe( struct net_deivce *dev)
 .
 . Purpose:
 .	Tests to see if a given device has a vsc452mac chip.
 .	Returns a 0 on success
 .-----------------------------------------------------------------
 */
static int __init
do_vsc452mac_probe(struct net_device *dev)
{
	int retval;
	struct vsc452mac_local  *lp;
	u8 mac_addr[6];
	static unsigned version_printed = 0;
	unsigned int baseAddr = dev->base_addr;
	uint32 value=0;
	uint32 port =0;

	if (version_printed++ == 0)
	printk("%s", version);

	/* Print out the device info, in a short format.. */
	printk("%s: device at %#3x IRQ:%d NOWAIT:%d\n",dev->name, baseAddr, dev->irq, dev->dma);

	lp = netdev_priv(dev);    
	memset(lp, 0 , sizeof(struct vsc452mac_local)); 
	lp->dev = dev;

	spin_lock_init(&lp->lock);

#ifdef VSC452MAC_USE_DMA
	//SET_MODULE_OWNER(dev);

	/* Allocate for both rx and tx descriptors (rx=128 and tx=32 total=160) */
	lp->rx_ring_desc = dma_alloc_coherent(NULL,
		(RX_MAX_DESC_COUNT * sizeof( struct dma_rx_desc_T )) +
		(TX_MAX_DESC_COUNT * sizeof(struct dma_tx_desc_T)),
		&lp->rx_ring_dma,
		GFP_KERNEL);

	/* If allocation fails return error */
	if (!lp->rx_ring_desc)
	{
		free_netdev(dev);
		printk(" DMA alloc failed.\n");
		return (-ENOMEM);
	}

	/* Point tx descriptor start to rx descriptor start + 128 */
	lp->tx_ring_desc = (struct dma_tx_desc_T *)(lp->rx_ring_desc + RX_MAX_DESC_COUNT);

	/* Point tx dma (physical address) to rx dma (physical address) + 128 */
	lp->tx_ring_dma = lp->rx_ring_dma + (RX_MAX_DESC_COUNT * sizeof(struct dma_rx_desc_T));

#endif
	/* Fill in the fields of the device structure with ethernet values. */
	ether_setup(dev);


	// dev->poll=vsc452mac_dma_receive;
	// dev->weight = RX_MAX_DESC_COUNT;
	netif_napi_add(dev, &lp->napi, vsc452mac_dma_receive, RX_MAX_DESC_COUNT);
	dev->hard_start_xmit = vsc452mac_dma_send_packet;

	dev->open	        	= vsc452mac_open;
	dev->stop	        	= vsc452mac_close;
	dev->do_ioctl			= vsc452_ioctl;
	//dev->tx_timeout		= vsc452mac_timeout;
	dev->get_stats			= vsc452mac_query_statistics;
	dev->set_mac_address = put_mac;
#ifdef	HAVE_MULTICAST
	dev->set_multicast_list = &vsc452mac_set_multicast_list;
#endif


	if ( baseAddr == 0xB0480000 )
		port = 1;
	else
		port = 0;

	/* set the mac address */
	// FindMACAddress(port,mac_addr);
	get_mac(dev->base_addr, mac_addr);
	put_mac(dev,mac_addr);
	vsc452_print_mac(dev->base_addr,NULL);


	spin_lock_irq(&lp->lock);

	/* now, reset the chip, and put it into a known state */
	vsc452mac_reset( dev );


	spin_unlock_irq(&lp->lock);


	retval = register_netdev(dev);
	if (retval)
	{

#ifdef VSC452MAC_USE_DMA
		dma_free_coherent(NULL,
				  RX_MAX_DESC_COUNT + RX_MAX_DESC_COUNT,
				  lp->rx_ring_desc,
				  lp->rx_ring_dma);
#endif
		free_netdev(dev);
		return retval;
	}

	/* turn of the netif, enabled in open function */
	netif_carrier_off(dev);

	/*enable MDIO baseed on port number */
	pioLd32(VSC452_OCR,&value);
	value |= (MDIOENA << port);
	pioSt32(VSC452_OCR,value);

	return 0;
}



/*
 * Open and Initialize the board
 *
 * Set up everything, reset the card, etc ..
 *
 */
static int
vsc452mac_open(struct net_device *dev)
{
	int err = 0;
	struct vsc452mac_local *lp = netdev_priv(dev);
	uint32  base = dev->base_addr;
	//printk("vsc452mac_open\n");
    /* Grab the IRQ */
    err = request_irq(dev->irq, vsc452mac_interrupt, IRQF_SHARED, dev->name, dev);
    if (err)
    {
     	printk("%s: unable to get IRQ %d (irqval=%ld).\n", dev->name, dev->irq, (long int)err);
    	return -1;
    }

	/* do the nic initialization stuff, lock it before doing*/
	spin_lock_irq(&lp->lock);

	pioSt32(base+REG_TRCR, TRCR_RMIIROS|TRCR_RCDR|TRCR_TCDR|TRCR_TFR|TRCR_RFR);
	udelay(5000);

	/*Mac control regsiter reset*/
	pioSt32(base+REG_MACC, 0);

#ifdef VSC452MAC_USE_DMA
	vsc452mac_enable_rx_dma(dev);

	/*Mac receive enable */
	pioSt32(base+REG_MAC1,MAC1_TFC|MAC1_RFC|MAC1_RE);

	/* enable DMA interrupts */
	pioSt32(base+REG_ISE, (TPCINTR|RPCINTR|RQOINTR|RBOIINTR|RFOIINTR));

#else

	/* Mac receive enable */
	pioSt32(base+REG_MAC1, MAC1_TFC|MAC1_RFC|MAC1_RE);

	/* enable FIFO interrupts */
	pioSt32(base+REG_ISE,(TSVPIINTR|RSVPIINTR|TFAFIINTR|RFOFINTR) );

#endif



	pioSt32(base+REG_TRCR , TRCR_RMIIROS);

	spin_unlock_irq(&lp->lock);

	/* phy configuration*/
	vsc452mac_phy_configure(dev);

	netif_carrier_on(dev);
	napi_enable(&lp->napi);
	netif_start_queue(dev);

	return 0;
}


/*--------------------------------------------------------
 . Called by the kernel to send a packet out into the void
 . of the net.  This routine is largely based on
 . skeleton.c, from Becker.
 .--------------------------------------------------------
*/
#if 0
static void
vsc452mac_timeout (struct net_device *dev)
{
    /* If we get here, some higher level has decided we are broken.
    There should really be a "kick me" function call instead. */
    printk(KERN_WARNING "%s: transmit timed out?\n",dev->name);


    vsc452mac_reset( dev );
    vsc452mac_enable( dev );
    vsc452mac_phy_configure(dev);
    netif_wake_queue(dev);
    dev->trans_start = jiffies;
}
#endif
/*--------------------------------------------------------------------
 .
 . This is the main routine of the driver, to handle the net_device when
 . it needs some attention.
 .
 . So:
 .   first, save state of the chipset
 .   branch off into routines to handle each case, and acknowledge
 .	    each to the interrupt register
 .   and finally restore state.
 .
 ---------------------------------------------------------------------*/

/*Ioctl function for loopback test*/
static int vsc452_ioctl (struct net_device *dev, struct ifreq *rq, int cmd)
{
	u32 baseAddr;
	char  tx_buffer[BUFLEN];
	struct vsc452_ioctl *ioc = (struct vsc452_ioctl * )rq->ifr_ifru.ifru_data;
	struct sk_buff *sk_buf;
	baseAddr = vsc452mac_io[ioc->port];
	switch(ioc->cmd)
	{
		case VSC452_ENABLE_INTERNAL_LOOPBACK:           /* Enable internal loopback */

				/*Enabling internal loopback sequence
 *				1. disable receive enable
 *				2. clear the statistics
 *				3. enable loopback
 *				4. clear buffers
 *				5. init ring buffers
 *				6. enable receive
 *				the following are the steps for loopback test.
 *				7. send data
 *				8. receive
 * 				*/

				pioSt32 ( baseAddr + REG_MAC1, 0x0000cf00 );
				pioSt32 ( baseAddr + REG_SUPP, 0x08888 );
				pioSt32 ( baseAddr + REG_MCFG, 0x08000 );
				vsc452mac_disable_rx_dma(dev);
   				vsc452mac_disable_tx_dma(dev);
				udelay(5000);
//				vsc452mac_dma_free_buffers(dev);
				vsc452mac_reinitbuffers(dev);
				vsc452mac_dma_ringsetup(dev);
				//initialize mac and phy values
				pioSt32 ( baseAddr + REG_MAC1, 0x00000010 );
				pioSt32 ( baseAddr + REG_SUPP, SUPP_SPEED );
				pioSt32 ( baseAddr + REG_MCFG, 0x0018 );
				vsc452mac_enable_rx_dma(dev);
				vsc452mac_enable_tx_dma(dev);
				printk(KERN_NOTICE" Internal loop back is enabled for port = %d \n",ioc->port);
				break;
		case VSC452_DISABLE_INTERNAL_LOOPBACK:           /* Disable interanal loopback */
				/*disbling loopback sequence*/
				/*1. disable the loopback bit
 * 				  2. no need to touch the ring and dma buffers.
 * 				*/
				pioSt32 ( baseAddr + REG_MAC1, 0x0000cf00 );
				pioSt32 ( baseAddr + REG_SUPP, 0x08888 );
				pioSt32 ( baseAddr + REG_MCFG, 0x08000 );
				udelay(5000);
				//initialize mac and phy values
				pioSt32 ( baseAddr + REG_MAC1, MAC1_TFC|MAC1_RFC|MAC1_RE);
				pioSt32 ( baseAddr + REG_SUPP, SUPP_SPEED );
				pioSt32 ( baseAddr + REG_MCFG, 0x0018 );

				printk(KERN_NOTICE " Internal loop back is disabled  for port = %d \n",ioc->port);
				break;
		case  LOOPBACK_TEST  :    /* Send Data  */
				enableLoopback=1;
				/*get transmit data froom the userspace */
				if (copy_from_user(&tx_buffer, ioc->tx_data,BUFLEN)) return -EFAULT;
				/*alloacte memory for sk_buff*/
			        sk_buf = dev_alloc_skb(MAX_ETH_FRAME_SIZE);
				if (sk_buf ==NULL)
			        	{
						printk(KERN_NOTICE "%s: Low memory, packet dropped.\n",	dev->name);
						return ( -ENOMEM);
			        	}
				sk_buf->data=(char*)tx_buffer;
				sk_buf->len=strlen(tx_buffer);

				/*clear the buffer*/
				memset (&rx_buffer[0],0,BUFLEN);
				/*sending data */
				vsc452mac_dma_send_packet(sk_buf,dev);
                               udelay(50);
				/*send the receive data to user space stored from the global array (rx_buffer)
				     in  "vsc452mac_dma_receive"*/
				if (copy_to_user((void*)ioc->rx_data,(void*)&rx_buffer,BUFLEN))return -EFAULT;

				/* send the netsats to userspace*/
				//if (copy_to_user((void*)&ioc->app_stats,(void*)&lp->stats,sizeof( struct net_device_stats) ))return -EFAULT;
				//printk(" Inside the Loopback Test \n");
				ioc->app_stats->tx_packets = 1;
				ioc->app_stats->rx_packets =1;
				ioc->app_stats->tx_bytes = BUFLEN;
				ioc->app_stats->rx_bytes = BUFLEN;
				ioc->app_stats->rx_errors =0;
				ioc->app_stats->tx_dropped =0;
				ioc->app_stats->rx_dropped =0;
				ioc->app_stats->collisions =enableLoopback;

				break;
		default:
				return -EOPNOTSUPP;
	}
	
	return 0;
}

static irqreturn_t
vsc452mac_interrupt(int irq, void * dev_id)
{
	struct net_device *dev = (struct net_device*)dev_id;
	unsigned int  baseAddr = dev->base_addr;
	uint32	status = 0;			// interrupt status
	struct vsc452mac_local  *lp = netdev_priv(dev);
	int res =0;
	unsigned long flags = 0;
	int dma_intr = 0;
	u32 ior0_val;

	if (dev == NULL)
	{
		printk(KERN_WARNING "%s: irq %d for unknown device.\n",	dev->name, irq);
		return IRQ_NONE;
	}

	dma_intr = (RPCINTR|RQOINTR|RBOIINTR|RFOIINTR|TPCINTR);

	/* Read the interrupt status registers */
	spin_lock_irqsave(&lp->lock, flags);
	ior0_val = *IOR0_REG;
	*HIF_REG = 0; /* thanks Bala this worked! */
	spin_unlock_irqrestore(&lp->lock, flags);

	/* Make sure interrupt is for NICs with proper base address specified */
	if (((baseAddr == VSCMAC_BASE0) && ((ior0_val & IOR0_EMACIS0_BITMASK) == 0)) ||
		((baseAddr == VSCMAC_BASE1) && ((ior0_val & IOR0_EMACIS1_BITMASK) == 0)))
	{
		return IRQ_NONE;
	}

	pioLd32( baseAddr + REG_IS,  (uint32*)&(lp->intrstatus));
	pioLd32( baseAddr + REG_ISE, (uint32*)&(lp->ierstatus));

	status = lp->intrstatus & lp->ierstatus;
	//intk("intrstatus %x  enable %x curint %lx\n",lp->intrstatus,lp->ierstatus,status);


	spin_lock_irqsave(&lp->lock, flags);


#ifdef VSC452MAC_USE_DMA
	/* handle Transmit interrupts*/
	if (status & (TPCINTR))
	{
       	res = vsc452mac_dma_int_tx(dev);
	}

	/* handle receive interrupts */
	if (status & (RPCINTR|RQOINTR|RBOIINTR|RFOIINTR))
	{
		 res = res | vsc452mac_dma_int_rx(dev, &lp->napi);
	}
	if (status & PHYINTR)
	{
		/* dont do anything */
	}


	lp->intrstatus &= (~(dma_intr|PHYINTR));
	lp->intrstatus |= res;
	/* clear the status register */
	pioSt32(baseAddr+REG_IS,lp->intrstatus);

#else
	//printk("I= 0x%x\n",status);

	/* If Fifo Empty Return */
	if (status & IS_RFE)
	{
		pioSt32(baseAddr+REG_IS,status);
		return IRQ_HANDLED;
	}

	//if (status & IS_RGP)
	//	printk("Received Good\n");
	//
	//if (status & IS_RFOR)
	//	printk("Received FIFO Out ready\n");

	/* Almost full. To avoid fifo overrun, stop
	   receive and schedule task to process all data */
	if (status & IS_RFO)
	{
		printk("vsc425mac:Overflow During Interrupt\n");
#if FIFO_CLR_ON_OVERFLOW
		/* Fix for the Bug */
		/* Disable recevier, flush fifo and renable receiver*/
	    	pioSt32 ( baseAddr + REG_MAC1, 0);
    		pioSt32 ( baseAddr + REG_TRCR, TRCR_RMIIROS|TRCR_RFR|TRCR_RCDR);
    		pioSt32 ( baseAddr + REG_TRCR, TRCR_RMIIROS);
    		pioSt32 ( baseAddr + REG_MAC1, MAC1_RE );
#endif
        	lp->stats.rx_errors++;
	}

	if (status & IS_RFAF)
	{
		if (lp->trans_busy == 0)
		{
    			pioSt32 ( baseAddr + REG_MAC1,0);
    			pioSt32 ( baseAddr + REG_ISE,0);
		//	printk("VSC452Mac :Disabling Recv ...\n");
			printk("Dis\n");
       		lp->trans_busy = 1;
			INIT_WORK(&lp->rcv_work,vsc452mac_rcv,dev);
			schedule_work(&lp->rcv_work);
		}
		else
			printk("VSC452Mac: Already scheduled 0x%lx\n",status);
		pioSt32(baseAddr+REG_IS,status);
    	return IRQ_HANDLED;
	}

	/* Not Empty and not almost full. Do normal process */
   	vsc452mac_rcv(dev);
#endif


	spin_unlock_irqrestore(&lp->lock, flags);
	return IRQ_HANDLED;
}



#ifndef VSC452MAC_USE_DMA
/*
 . Function: vsc452mac_wait_to_send_packet( struct sk_buff * skb, struct device * )
 . Purpose:
 .    Attempt to allocate memory for a packet, if chip-memory is not
 .    available, then tell the card to generate an interrupt when it
 .    is available.
 .
 . Algorithm:
 .
 . o	if the saved_skb is not currently null, then drop this packet
 .		on the floor.  This should never happen, because of TBUSY.
 . o	if the saved_skb is null, then replace it with the current packet,
 . o	See if I can sending it now.
 . o 	(NO): Enable interrupts and let the interrupt handler deal with it.
 . o	(YES):Send it now.
*/
static int
vsc452mac_wait_to_send_packet( struct sk_buff * skb, struct net_device * dev )
{
	struct vsc452mac_local  *lp = netdev_priv(dev);
	unsigned int baseAddr = dev->base_addr;
	int  pktLen;
	unsigned long flags;
	unsigned long status;
	uint32* dataPtr;
	int timeout;

	spin_lock_irqsave(&lp->lock, flags);
	if (skb==NULL)
	{
		printk("%s(%d): NULL skb???\n", __FILE__,__LINE__);
		spin_unlock_irqrestore(&lp->lock, flags);
		return 0;
	}
	//printk("vsc452mac_send\n");

	pktLen = ETH_ZLEN < skb->len ? skb->len : ETH_ZLEN;
	if ( ( pktLen < 8 ) || ( pktLen > MAX_ETH_FRAME_SIZE ) )
	{
		printk("%s(%d): skb len invalid = %d\n", __FILE__,__LINE__,pktLen);
		spin_unlock_irqrestore(&lp->lock, flags);
		return 0;
	}

	dataPtr = (uint32 *)(skb->data);

	// pad out to the nearest 8 byte boundary
	if (pktLen & 0x3 )
	{
		pktLen += 8;
		pktLen &= ~0x3;
	}

#if VSC452MAC_DEBUG > 2
	printk("VSC452MAC: Sending Packet of size %d\n",skb->len);
	print_packet( skb->data, pktLen);
#endif

	// place first packet (dword) in the FIFO
	pioSt32 ( baseAddr + REG_TFC, TFC_SOP );
	pioSt32 ( baseAddr + REG_TFD, *dataPtr );
	pktLen -= 4;
	dataPtr++;

	// place middle packets (dwords) in the FIFO
	pioSt32 ( baseAddr + REG_TFC, TFC_MID );
	while ( pktLen > 4 )
	{
		pioSt32 ( baseAddr + REG_TFD, *dataPtr );
		pktLen -= 4;
		dataPtr++;
	}

	// place last packet (dword) in the FIFO
	switch ( pktLen )
	{
        case 1 :
                pioSt32 ( baseAddr + REG_TFC, TFC_EOP1 );
                break;
        case 2 :
                pioSt32 ( baseAddr + REG_TFC, TFC_EOP2 );
                break;
        case 3 :
                pioSt32 ( baseAddr + REG_TFC, TFC_EOP3 );
                break;
        case 4 :
                pioSt32 ( baseAddr + REG_TFC, TFC_EOP4 );
                break;
        default :
				printk("%s(%d): Unknown code pktlen= %d\n", __FILE__,__LINE__,pktLen);
                break;
	}
	pioSt32 ( baseAddr + REG_TFD, *dataPtr  );

	// Transmit the packets
	pioSt32 ( baseAddr + REG_TRCR, TRCR_RMIIROS|TRCR_TSFS );

	status =0;
	timeout = 0;
	while (!(status & IS_TFAE))
	{
		pioLd32( baseAddr + REG_IS, &status );
		// TODO: Sleep for the interrupt instead looping!!
		timeout++;
		if (!(timeout % 100))
			printk("x");
	}

	pioSt32 ( baseAddr + REG_TRCR, TRCR_RMIIROS);

	lp->stats.tx_packets++;
	dev_kfree_skb_any (skb);
	dev->trans_start = jiffies;
	spin_unlock_irqrestore(&lp->lock, flags);
	return 0;
}




static
int
RcvSinglePacket ( uint32 baseAddr, uint8 *pktData )
{
	uint32 *buf = (uint32*)pktData;
	uint32 data=0,expected=0;
	uint32 code;
	uint32 done;
	unsigned long pktLen = 0;

	/* Get Last Received Byte Count */
	pioLd32 ( baseAddr + REG_RSV, &expected );


	done   = 0;
	pktLen = 0;

	do
	{
		/* Wait if  no data is avail */  /* TODO */

   		// read in the data from FIFO
       	 pioLd32 ( baseAddr + REG_RFD, &data );
		//if (pktLen <=  MAX_ETH_FRAME_SIZE)
        	 *buf = data;
		/*	else
		{
			printk("RcvSinglePacket : Too Big Packet %d !!!\n",pktLen);
			break;
		}*/

        	// process the corresponding code
        	pioLd32 ( baseAddr + REG_RFC, &code );
        	switch ( code )
        	{
                case RFC_SOP :
                    buf += 1;
                    pktLen += 4;
                    break;
                case RFC_MID :
                    buf += 1;
                    pktLen += 4;
                    break;
                case RFC_EOP1 :
                    pktLen += 1;
                    done = 1;
                    break;
                case RFC_EOP2 :
                    pktLen += 2;
                    done = 1;
                    break;
                case RFC_EOP3 :
                    pktLen += 3;
                    done = 1;
                    break;
                case RFC_EOP4 :
                    pktLen += 4;
                    done = 1;
                    break;
                default :
                    printk("RcvSinglePacket: RFC  error, code=0x%lx\n",code );
                    done = 1;
                    break;
            	}
	} while ( !done );

	//	if ((expected&0xffff) != pktLen)
	//	{
	//		printk("Expected : 0x%lx \n",expected& 0xFFFF);
	//		printk("Got : 0x%lx \n",pktLen);
	//	}

	printk( "returning from receive %d\n",pktLen);
	return ( pktLen );
}



/*-------------------------------------------------------------
 .
 . vsc452mac_rcv -  receive a packet from the card
 .
 . There is ( at least ) a packet waiting to be read from
 . chip-memory.
 .
 . o Read the status
 . o If an error, record it
 . o otherwise, read in the packet
 --------------------------------------------------------------
*/

static void
vsc452mac_rcv(void *devp)
{
	struct net_device *dev = (struct net_device *)devp;
	struct vsc452mac_local *lp = netdev_priv(dev);
	unsigned int baseAddr = dev->base_addr;
	int pktLen;
	char *skbdata;
	char rxBuff[MAX_ETH_FRAME_SIZE];
	unsigned long data;
	struct sk_buff  * skb;
	int i;

	/* Maximum packets = BufferSize/MinFrameSize */
	for(i=0;i<((8*1024)/MIN_ETH_FRAME_SIZE);i++)
	{
		//printk("r");
		pioLd32( baseAddr + REG_IS, &data );
		pioSt32(baseAddr + REG_IS, data);
		if (data & IS_RFO)
		{
			printk("vsc425mac:Overflow During Processing\n");
			if (lp->trans_busy)
				printk("BAD!!!!!!!. Overflow during Bottom Halt!!!!!!!!!!!!!!!!!!\n");
#if FIFO_CLR_ON_OVERFLOW
			/* Fix for the Bug */
			/* Disable recevier, flush fifo and renable receiver*/
    			pioSt32 ( baseAddr + REG_MAC1, 0);
    			pioSt32 ( baseAddr + REG_TRCR, TRCR_RMIIROS|TRCR_RFR|TRCR_RCDR);
    			pioSt32 ( baseAddr + REG_TRCR, TRCR_RMIIROS);
    			pioSt32 ( baseAddr + REG_MAC1, MAC1_RE );
#endif
            		lp->stats.rx_errors++;
			//continue;
		}

		if (data & IS_RFE)
			break;
		pktLen = RcvSinglePacket(baseAddr, rxBuff);
		if(pktLen <=4)
		{
       		lp->stats.rx_errors++;
			printk("EOF len = %d\n",pktLen);
			break;
		}
		if (pktLen > MAX_ETH_FRAME_SIZE)
		{
		        lp->stats.rx_errors++;
			printk("TOO BIG pktLen = %d\n",pktLen);
			continue;
		}
		if (pktLen < MIN_ETH_FRAME_SIZE)
		{
           		lp->stats.rx_errors++;
			printk("TOO SMALL PktLen = %d\n",pktLen);
			continue;
		}

		/* Valid Packet - Copy to skb */

		// Allocate enough memory for entire receive frame, to be safe
		skb = dev_alloc_skb( pktLen+2 );

		if ( skb == NULL )
		{
			printk(KERN_NOTICE "%s: Low memory, packet dropped.\n",	dev->name);
			lp->stats.rx_dropped++;
			continue;
		}

		skb_reserve( skb, 2 );   /* 16 bit alignment */
       	skb->dev = dev;
       	skbdata = skb_put( skb, pktLen);
       	memcpy(skbdata,rxBuff,pktLen);
      		skb->protocol = eth_type_trans(skb, dev );
	   	printk("sending it to net interface\n");
       	netif_rx(skb);
       	lp->stats.rx_packets++;
	}

	if (lp->trans_busy == 1)
	{
		// printk("VSC452Mac :ReEnabling Recv ...\n");
		printk("Ena\n");
		lp->trans_busy = 0;
    		pioSt32 ( baseAddr + REG_ISE, ISE_RFAF|ISE_RSVP|ISE_RFOR|ISE_RGP|ISE_RFO);
    		pioSt32 ( baseAddr + REG_MAC1, MAC1_RE );
	}
	return;
}

#endif

/*----------------------------------------------------
 . vsc452mac_close
 .
 . this makes the board clean up everything that it can
 . and not talk to the outside world.   Caused by
 . an 'ifconfig ethX down'
 .
 -----------------------------------------------------*/
static int
vsc452mac_close(struct net_device *dev)
{
	struct vsc452mac_local  *lp = netdev_priv(dev);
	netif_stop_queue(dev);
	napi_disable(&lp->napi);
	spin_lock_irq(&lp->lock);

	vsc452mac_shutdown( dev->base_addr );
	if (dev->irq != 0)
		free_irq(dev->irq,dev);
	spin_unlock_irq(&lp->lock);

#ifdef MODULE
	// MOD_DEC_USE_COUNT;
#endif

	return 0;
}

/*------------------------------------------------------------
 . Get the current statistics.
 . This may be called with the card open or closed.
 .-------------------------------------------------------------*/
static struct net_device_stats*
vsc452mac_query_statistics(struct net_device *dev)
{
	struct vsc452mac_local  *lp = netdev_priv(dev);

//	printk("vsc452mac_query_statistics\n");

	return &lp->stats;
}


/******************************************************************
 *********************	MULTICAST ADDR FUNCTIONS ******************
 ******************************************************************/


#ifdef HAVE_MULTICAST

/*
 . Function: vsc452mac_setmulticast( unsigned int baseAddr, int count, dev_mc_list * adds )
 . Purpose:
 .    This sets the internal hardware table to filter out unwanted multicast
 .    packets before they take up memory.
*/

static void
vsc452mac_setmulticast( unsigned int baseAddr, int count, struct dev_mc_list * addrs )
{
	struct dev_mc_list * cur_addr;
	int crc_val;

	printk("+vsc452mac_setmulticast\n");

	for (cur_addr = addrs ; cur_addr!=NULL ; cur_addr = cur_addr->next )
	{
       	if ( !( *cur_addr->dmi_addr & 1 ) )
            		continue;
        	crc_val = crc32( cur_addr->dmi_addr, 6 );
        	crc_val = (crc_val>>26)&0x3f;			// ¨ú MSB 6 bit
        	if (crc_val >= 32)
            		outl(inl(baseAddr+MAHT1_REG) | (1UL<<(crc_val-32)), baseAddr+MAHT1_REG);
        	else
            		outl(inl(baseAddr+MAHT0_REG) | (1UL<<crc_val), baseAddr+MAHT0_REG);
	}
}


/*-----------------------------------------------------------
 . vsc452mac_set_multicast_list
 .
 . This routine will, depending on the values passed to it,
 . either make it accept multicast packets, go into
 . promiscuous mode ( for TCPDUMP and cousins ) or accept
 . a select set of multicast packets
*/
static void
vsc452mac_set_multicast_list(struct net_device *dev)
{
	unsigned int baseAddr = dev->base_addr;
	struct vsc452mac_local  *lp = netdev_priv(dev);

	printk("+vsc452mac_set_multicast_list\n");

	if (dev->flags & IFF_PROMISC)
       	 lp->maccr_val |= RCV_ALL_bit;
	else
        	 lp->maccr_val &= ~RCV_ALL_bit;

	if ( !(dev->flags & IFF_ALLMULTI) )
		lp->maccr_val |= RX_MULTIPKT_bit;
	else
        	lp->maccr_val &= ~RX_MULTIPKT_bit;

	if (dev->mc_count)
    	{
		lp->maccr_val |= HT_MULTI_EN_bit;
        	vsc452mac_setmulticast( baseAddr, dev->mc_count, dev->mc_list );
	}
    	else
       	 lp->maccr_val &= ~HT_MULTI_EN_bit;

    	outl( lp->maccr_val, baseAddr + MACCR_REG );
}
#endif



/******************************************************************
 *********************	PHY RELATED FUNCTIONS *********************
 ******************************************************************/

#if 0
/*------------------------------------------------------------
 . Reads a register from the MII Management serial interface
 .-------------------------------------------------------------*/
static unsigned short
vsc452mac_read_phy_register(unsigned int baseAddr,
				   unsigned char phyaddr, unsigned char phyreg)
{
	return 0;
}


/*------------------------------------------------------------
 . Writes a register to the MII Management serial interface
 .-------------------------------------------------------------*/
static void
vsc452mac_write_phy_register(unsigned int baseAddr,
		unsigned char phyaddr, unsigned char phyreg, unsigned short phydata)
{

}

#endif

/*---------------------------------------------------------------
 . Configures the specified PHY using Autonegotiation.
 .-------------------------------------------------------------*/
static void
vsc452mac_phy_configure(struct net_device* dev)
{

	struct vsc452mac_local *lp = netdev_priv(dev);
	u32 reg, adv;

	// printk("vsc452mac_phy_configure\n");

	/* advertise registers*/
	lp->phyaddr=VSC452_PHY_ADDR;
	adv = vsc452mac_readphy(dev, lp->phyaddr, MII_ADVERTISE);
	adv |= ADVERTISE_ALL;
	vsc452mac_writephy(dev, lp->phyaddr, MII_ADVERTISE, adv);

	/*auto negotiation */
	reg = vsc452mac_readphy(dev, lp->phyaddr, MII_BMCR);
	reg |= (BMCR_ANENABLE | BMCR_ANRESTART);
	vsc452mac_writephy(dev, lp->phyaddr, MII_BMCR, reg);
	return;
}

//following are the DMA functions.
#ifdef VSC452MAC_USE_DMA


static uint32	vsc452mac_dma_int_rx(struct net_device *dev, struct napi_struct *napi)
{
	struct vsc452mac_local *lp = netdev_priv(dev);
	uint32 int_status = 0;
	uint32 res = 0;

	/* get the intr status */
	int_status = lp->intrstatus;

	 /* buffer overflow intr*/
	if (int_status & RBOIINTR)
	{
		// printk( "rx dma buffer overrun\n");
       	res |= RBOIINTR;
	}

	 /* receive packet complete intr */
	if (int_status & RPCINTR)
	{
		pioSt32((dev->base_addr+REG_ISE),RQOINTR|RBOIINTR|RFOIINTR);
	}

	/* fifo overflow intr */
	if (int_status & RFOIINTR)
	{
		// printk( "rx fifo overrun\n");
		lp->stats.rx_fifo_errors++;
		res |= RFOIINTR;
	}

	/* queue overrun intr */
	if (int_status & RQOINTR)
	{
		// printk( "rx dma queue overrun\n");
		res |= RQOINTR;
	}

	/* Schedule NAPI receive poller */
	netif_rx_schedule(dev, napi);

	//printk ( "Exiting dma int rx\n");
	return res;
}


static int vsc452mac_dma_receive(struct napi_struct *napi, int budget)
{
	struct sk_buff *skb;
	struct dma_rx_desc_T *rx_ring;
	struct vsc452mac_local *lp = container_of(napi, struct vsc452mac_local, napi);
	struct net_device *dev = lp->dev;
	uint32 baddr = dev->base_addr;
	int cur_entry = 0;
	int work_done = 0;
	int pkts_received = 0;
	int pkt_len = 0;
	uint32 flags = 0;
	uint32 temp = 0;
	static int prev = 0;
	uint32 clear = 0;
	int budgetreached=0;
	// uint32 i = 0;
	/* get the current rx*/
	cur_entry = lp->rx_front & RX_RING_MASK;

	/* do until there is no receive packet comple te interrupt */
	while (1)
	{
    		/*clear the interrupt */
    		clear |= RPCINTR;
		pioSt32(baddr+REG_IS,clear);

		while (lp->rx_ring_desc[cur_entry].owner == OWNER_HOST)
        	{
			/* no packets received*/
            		if ((lp->rx_rear + RX_MAX_DESC_COUNT)== lp->rx_front)
            		{
            			printk ( "No active buffers\n");
                		break;
           		}

			/* is budget limit reached*/
            		if (++work_done >= budget)
            		{
        			// printk("pkt limit reached %d\n",*budget);
          			    budgetreached=1;
                		break;
            		}
#if 0
	    		/* do the packet filtering based on our mac address */
	    		for ( i=0;i<6;i++)
	    		{
				if(lp->rx_buf[cur_entry].sk_buf->data[i]!= dev->dev_addr[i])
				{
		   			rx_ring->ok = 0;
		    			break;
				}
	    		}
#endif
	    		lp->rx_buf[cur_entry].sk_buf->len = pkt_len;
            		skb = lp->rx_buf[cur_entry].sk_buf;
#if 0
	    		for ( i=0;i<6;i++)
	    		{
				printk("%2x:",skb->data[i]);
	    		}
	    		printk("\n");
#endif

            		rx_ring = &lp->rx_ring_desc[cur_entry];
            		pkt_len = rx_ring->rxcnt;

			/* check for valid packet other wise update error statistics*/
            		if(!rx_ring->orunerr
                		&& rx_ring->ok
                		&& (pkt_len >= ETH_ZLEN)
                		&& (pkt_len <= MAX_ETH_UNTAGGED_FRAME_SIZE)
                		&& (rx_ring->fifo_rxcnt == pkt_len))
            		{

				skb->dev = dev;
               		/* 2 bytes for alignment */
               		skb_reserve(skb, 2);
                		skb_put(skb, pkt_len);


                		/* unmap skb */
                		dma_unmap_single(NULL,
                               		 lp->rx_buf[cur_entry].skb_dma,
                                 		 MAX_ETH_FRAME_SIZE,
                                 		 DMA_FROM_DEVICE);

                		/* reset the buffer info */
                		lp->rx_buf[cur_entry].sk_buf = NULL;
                		lp->rx_buf[cur_entry].skb_dma = 0;

				/*Loopback test*/
				if (enableLoopback==1)
				{
					/*store the receiived data in the global array rx_buffer*/
					memcpy(rx_buffer,skb->data,strlen(skb->data));
					enableLoopback=0;
				}
                		/* send the data to kernel for processing */
            			skb->dev = dev;
                		skb->protocol = eth_type_trans(skb, dev);
                		skb->ip_summed = CHECKSUM_UNNECESSARY;

				prev = netif_receive_skb(skb);

                		/* Update statistics */
                		dev->last_rx = jiffies;
                		lp->stats.rx_packets++;
                		lp->stats.rx_bytes += pkt_len;
                		if (rx_ring->multicast)
                		{
                    			lp->stats.multicast++;
                		}
			}
            		else
            		{
       			if (rx_ring->crcerr)
                		{
                    			lp->stats.rx_crc_errors++;
				}
               		 if (rx_ring->len_chkerr)
                		{

                    			lp->stats.rx_length_errors++;
                		}
                		if (rx_ring->orunerr)
                		{
					lp->stats.rx_over_errors++;
                		}
            		}

            		/* Go to next entry */
            		pkts_received++;
			/*increment the queue */
			++lp->rx_front;
            		cur_entry = lp->rx_front & RX_RING_MASK;

            		/* Refill rx buffers if more than 1/4 of the buffers are empty  */
           		if ((lp->rx_front - lp->rx_rear) >= (RX_MAX_DESC_COUNT/4))
            		{
                		vsc452mac_reinitbuffers(dev);
            		}
        	}
		pioLd32(baddr+REG_IS,&temp);
		if ( !(temp & RPCINTR) ) break;
	}

	/* Refill any remaining rx buffers */
	vsc452mac_reinitbuffers(dev);

	/* check the memory availability*/
	if (lp->rx_buf[lp->rx_rear & RX_RING_MASK].sk_buf == NULL)
	{
	     printk("dma receive mem alloc failed\n");
	}

	/* Enable the interrupt*/
	if(budgetreached==1)
		goto notdone;
	
	spin_lock_irqsave(&lp->lock, flags);
	netif_rx_complete(dev, napi);
	pioSt32((dev->base_addr+REG_ISE),(TPCINTR|RPCINTR|RQOINTR|RBOIINTR|RFOIINTR));
	spin_unlock_irqrestore(&lp->lock, flags);

	return work_done;
notdone:
	/* Start timer, stop polling, but do not enable rx interrupts. */
	udelay(jiffies+1);
	
	spin_lock_irqsave(&lp->lock, flags);
	/* remove ourselves from the polling list */
	netif_rx_complete(dev, napi);
	spin_unlock_irqrestore(&lp->lock, flags);
	
	/*Mask Recevice intrs and add the device to poll list. */
	vsc452mac_dma_int_rx(dev, &lp->napi);
 	return 0;

}


static int vsc452mac_reinitbuffers(struct net_device *dev)
{
	struct vsc452mac_local *lp = netdev_priv(dev);
	int cur_entry =0;
	int refill = 0;
	uint32 temp =0;

	/* Refill the receive ring skbs. */
	for (; lp->rx_front - lp->rx_rear > 0; lp->rx_rear++)
	{

		/* If the current unused entry does not have an skb, allocate one */
       	cur_entry = lp->rx_rear & RX_RING_MASK;

		if( lp->rx_ring_desc[cur_entry].owner == OWNER_DMA )
		{
			printk ( "owner is dma , dont free this \n");
			break;
		}

       	if (lp->rx_buf[cur_entry].sk_buf == NULL)
        	{
            		if ((lp->rx_buf[cur_entry].sk_buf = dev_alloc_skb(MAX_ETH_FRAME_SIZE)) != NULL)
            		{
                		/* Map skb data area to physical address for use by DMA engine */
                		lp->rx_buf[cur_entry].skb_dma =
                    		dma_map_single(NULL,
                                   		lp->rx_buf[cur_entry].sk_buf->data,
                               		MAX_ETH_FRAME_SIZE,
                                  		DMA_FROM_DEVICE);


         		      	/* Mark as being used by this device. */
                		lp->rx_buf[cur_entry].sk_buf->dev = dev;

           		 }
            		else
            		{
                		/* Out of memory */
				printk("malloc failing in reinint buffers\n");
                		break;
            		}
            		refill++;
        	}


	        /* Update ring entry with new skb data address and set owner to DMA */
       	lp->rx_ring_desc[cur_entry].rxaddr = lp->rx_buf[cur_entry].skb_dma;
        	lp->rx_ring_desc[cur_entry].orunerr = FALSE;
		/* avoid compiler optimizations and order of execution is ensured*/
        	wmb();
		/*Change the owner so that dma will use this */
        	lp->rx_ring_desc[cur_entry].owner = OWNER_DMA;
	}

	pioLd32((dev->base_addr+RDCREG),&temp);
	if (( temp & RDSMASK ) == 0x00010000 )
	{
		vsc452mac_enable_rx_dma(dev);
	}
	return refill;
}


static int vsc452mac_dma_send_packet(struct sk_buff *skb, struct net_device *dev)
{

	struct vsc452mac_local *lp = netdev_priv(dev);
	int cur_entry = 0;
	struct vscmac_dma_T *tx_buff;
	struct dma_tx_desc_T *tx_ring;
	uint32 intr_enable = FALSE;
	uint32 flags = 0;

	//printk (KERN_INFO "vsc dma send PACKET\n");
	spin_lock_irqsave(&lp->lock, flags);


	/* Get the current free descriptor*/
	cur_entry = lp->tx_front & TX_RING_MASK;
	tx_buff = &lp->tx_buf[cur_entry];
	tx_ring = &lp->tx_ring_desc[cur_entry];


	/* map the received skb to ring buffer */
	tx_buff->skb_dma = dma_map_single(NULL,
										  skb->data,
										  skb->len,
										  DMA_TO_DEVICE);
	tx_buff->sk_buf = skb;
	tx_buff->sk_buf->len=skb->len;


	/* update the  descriptors */
	tx_ring->txaddr = tx_buff->skb_dma;
	tx_ring->pkt_txcnt = skb->len;


	intr_enable = FALSE;
	/* Enable the itnerrupt when we have reached the threshold  */
	if (((lp->tx_front - lp->tx_rear) & TX_RING_MASK) == TX_THRESHOLD )
	{
		intr_enable = TRUE;
	}
	else if (lp->tx_front - lp->tx_rear == (TX_MAX_DESC_COUNT - 2))
	{
		intr_enable = TRUE;
		netif_stop_queue(dev);
	}

	tx_ring->intrenable = intr_enable;

	/* avoid compiler optimizations and order of execution is ensured*/
	wmb();

	/*Change the owner so that dma will use this */
	tx_ring->owner = OWNER_DMA;

	/* Increment the current tx */
	lp->tx_front++;

	/* resume transmit  */
	vsc452mac_enable_tx_dma(dev);
	spin_unlock_irqrestore(&lp->lock, flags);

	dev->trans_start = jiffies;
	return 0;

}

static uint32	vsc452mac_dma_int_tx(struct net_device *dev)
{
	struct vsc452mac_local *lp = netdev_priv(dev);
	uint32 baddr = dev->base_addr;
	struct vscmac_dma_T *tx_buff;
	struct dma_tx_desc_T   *tx_ring;
	unsigned int tx_rear;
	uint32 int_status = 0;
	int cur_entry =0;
	int res = 0;


	/* Handle transmit queue overrun intr*/
	if (lp->intrstatus & TXQOINTR)
	{
	   res |= TXQOINTR;
	}

	/* Handle packet transmit complete interrupt */
	if (lp->intrstatus & TPCINTR)
	{
    		while (1)
    		{
    			pioSt32((baddr+REG_IS),TPCINTR);
    			for ( tx_rear = lp->tx_rear; lp->tx_front - tx_rear > 0; tx_rear++)
    			{
       			cur_entry = tx_rear & TX_RING_MASK;
				tx_ring = (struct dma_tx_desc_T   *)&lp->tx_ring_desc[cur_entry];

				/* still the packet is owned by dma, dont disturb */
				if (tx_ring->owner == OWNER_DMA )
				{
					break;
				}
				/* error statistics- may be not complete */
				if(tx_ring->done)
				{
					lp->stats.tx_packets++;
                    			lp->stats.tx_bytes += tx_ring->tsv_txcnt;
                    			lp->stats.collisions += tx_ring->colcnt;
				}

				/* Update statistics if any errors occurred */
			   	if ( *((uint32*)tx_ring) &  0xF030)
			   	{
					lp->stats.tx_errors++;
			   	}

				/* free  the skb (which is sent from the kernel ) */
				tx_buff = &lp->tx_buf[cur_entry];
				if ( tx_buff->sk_buf )
				{
					/* unmap it before freeing */
					dma_unmap_single(NULL,
                                     				tx_buff->skb_dma,
                                     				tx_buff->sk_buf->len,
                                     				DMA_TO_DEVICE);
                    			dev_kfree_skb_irq(tx_buff->sk_buf);

					/*reset the buffer info */
					tx_buff->sk_buf = NULL;
					tx_buff->skb_dma = 0;
				}

    			}

			/* Update the dirty buffer pointer */
    			lp->tx_rear = tx_rear;
    			pioLd32((baddr+REG_IS),&int_status);
			if (!(int_status & TPCINTR))
				break;
		}
	}

	/*  check whether ifc queue is stopped,if it then wake up the queue. */
	if (netif_queue_stopped(dev) && (lp->tx_front - lp->tx_rear < (TX_MAX_DESC_COUNT- 2)))
	{
		netif_wake_queue(dev);
	}
	return res;
}




/* Enables RX DMA */
static void vsc452mac_enable_rx_dma(struct net_device *dev)
{
	uint32 base = dev->base_addr;
	pioSt32((base+RDCREG),RDE | RDR | TBPM);
}

/* Enables TX DMA */
static void vsc452mac_enable_tx_dma(struct net_device *dev)
{
	uint32 base = dev->base_addr;
	pioSt32( (base+TDCREG), TDE | TDR);
}

/* Disables RX DMA */
static void vsc452mac_disable_rx_dma(struct net_device *dev)
{
	uint32 base = dev->base_addr;
	pioSt32((base+RDCREG),0);
}

/* Disables TX DMA */
static void vsc452mac_disable_tx_dma(struct net_device *dev)
{
	uint32 base = dev->base_addr;
	pioSt32( (base+TDCREG), 0);
}




/* initialize the dma

   1. set dma tx descriptor base address
   2. set dma rx decriptor base address
   3. set buffer max size to RDBSREG

*/
static void vsc452mac_init_dma(struct net_device *dev)
{
	struct vsc452mac_local *lp = netdev_priv(dev);

	uint32 base = dev->base_addr;

	lp->intrstatus = 0;
	lp->ierstatus = 0;

	// disable tx dma
	pioSt32((base+TDCREG),0);
	//set descriptor base address
	pioSt32((base+TDDBAREG), lp->tx_ring_dma);

	// disablerx dma
	pioSt32((base+RDCREG),0);
	//set descriptor base address
	pioSt32((base+RDDBAREG), lp->rx_ring_dma);

	//receive buffer max size
	pioSt32((base+RDBSREG), MAX_ETH_UNTAGGED_FRAME_SIZE + 2);

}



/*

 Allocate RX buffers.

*/
static uint32	vsc452mac_dma_alloc_buffers(struct net_device *dev)
{

	struct vsc452mac_local *lp = netdev_priv(dev);
	int err = 0;
	int i;

	/*Allocate buffer and do dma_map for each entry and store it in dma address */
	for (i = 0; i < RX_MAX_DESC_COUNT; i++)
	{
		if ((lp->rx_buf[i].sk_buf = dev_alloc_skb(MAX_ETH_FRAME_SIZE)) != NULL)
		{
			/*  dma mapping */
			lp->rx_buf[i].skb_dma = dma_map_single(NULL,
					lp->rx_buf[i].sk_buf->data,
					MAX_ETH_FRAME_SIZE,
					DMA_FROM_DEVICE);

			lp->rx_buf[i].sk_buf->dev = dev;
		}
		else  /* failed in the middle of allocating */
		{
			/* deallocate buffers */
			for (--i; i >= 0 ;i--)
			{
				dma_unmap_single(NULL,
					lp->rx_buf[i].skb_dma,
					MAX_ETH_FRAME_SIZE,
					DMA_FROM_DEVICE);

				dev_kfree_skb_any(lp->rx_buf[i].sk_buf);
				lp->rx_buf[i].sk_buf = NULL;
				lp->rx_buf[i].skb_dma = 0;
			}
			err = -ENOMEM;  /* No memory */
			break;
		}
	}
       /* Initialise the TX dma_map */
	for (i = 0; i < TX_MAX_DESC_COUNT; i++)
	{
		lp->tx_buf[i].sk_buf = NULL;
		lp->tx_buf[i].skb_dma = 0;
	}        
	return err;
}

/*

 Free Tx and  Rx buffers
*/
static uint32	vsc452mac_dma_free_buffers(struct net_device *dev)
{
	struct vsc452mac_local *lp = netdev_priv(dev);
	int err = 0;
	int i;

	/* Unmap and free transmit skbs */
	for (i = 0; i < TX_MAX_DESC_COUNT; i++)
	{
		if (lp->tx_buf[i].sk_buf)
		{
			dma_unmap_single(NULL,
							 lp->tx_buf[i].skb_dma,
							 lp->tx_buf[i].sk_buf->len,
							 DMA_TO_DEVICE);
			dev_kfree_skb_any(lp->tx_buf[i].sk_buf);
			lp->tx_buf[i].sk_buf = NULL;
			lp->tx_buf[i].skb_dma = 0;
		}
	}

	/* Unmap and free receive skbs */
	for (i = 0; i < RX_MAX_DESC_COUNT; i++)
	{
		if (lp->rx_buf[i].sk_buf)
		{
			dma_unmap_single(NULL,
							 lp->rx_buf[i].skb_dma,
							 MAX_ETH_FRAME_SIZE,
							 DMA_FROM_DEVICE);
			dev_kfree_skb_any(lp->rx_buf[i].sk_buf);
			lp->rx_buf[i].sk_buf = NULL;
			lp->rx_buf[i].skb_dma = 0;
		}
	}

	return err;
}


/*

	Initialize the RX and TX descriptors.

*/
static uint32 vsc452mac_dma_ringsetup(struct net_device *dev)
{

	struct vsc452mac_local *lp = netdev_priv(dev);
	struct dma_rx_desc_T *rx_ring = lp->rx_ring_desc;
	struct dma_tx_desc_T *tx_ring = lp->tx_ring_desc;
	struct vscmac_dma_T *rx_buff = &lp->rx_buf[0];
	int i =0;

	/* Initialie current and dirty pointes */
	lp->rx_front = 0;
	lp->rx_rear = 0;

	/* Initialize the receive descriptors */
	for (i = 0; i < RX_MAX_DESC_COUNT; i++)
	{
	   rx_ring->eoq = FALSE;
	   rx_ring->orunerr = FALSE;
	   rx_ring->rxaddr = rx_buff->skb_dma;
	   /* avoid compiler optimizations and order of execution is ensured*/
	   wmb();

	   /*setting dma as owner so that dma will use this */
	   rx_ring->owner = OWNER_DMA;

	   /*increment the buffer and decriptor */
	   rx_buff++;
	   rx_ring++;
	}

	/* set end of queue marker */
	lp->rx_ring_desc[RX_MAX_DESC_COUNT - 1].eoq = TRUE;

	/* Initialize transmit ring */
	lp->tx_front = 0;
	lp->tx_rear = 0;

	/* Initialize the transmit descriptors */
	for (i = 0; i < TX_MAX_DESC_COUNT; i++)
	{
	   tx_ring->owner = OWNER_HOST;
	   /* avoid compiler optimizations and order of execution is ensured*/
	   wmb();
	   tx_ring->txaddr = 0;
	   tx_ring->pp_paden = TRUE;
	   tx_ring->pp_hugeen = FALSE;
	   tx_ring->pp_override = FALSE;
	   tx_ring->pkt_txcnt = 0;
	   tx_ring->eoq = FALSE;
	   tx_ring->intrenable = TRUE;
	   tx_ring->pp_crcen = TRUE;

	  /*increment the decriptor */
	   tx_ring++;
	}

	/* set end of queue marker */
	lp->tx_ring_desc[TX_MAX_DESC_COUNT - 1].eoq = TRUE;

	return 0;

}

#endif



/* ........................... PHY INTERFACE ........................... */
/*
 * Write value to the a PHY register
 */
static void vsc452mac_writephy(struct net_device *dev, uint8 phy_id, uint8 address, uint32 value)
{
	uint32 base = dev->base_addr;
	uint32 timeout = 200000;
	uint32 temp=0;

	pioSt32((base+REG_MCMD),0);
	pioSt32((base+REG_MADR),address | (phy_id << 8));
	pioSt32((base+REG_MWTD),value);

	do
	{
		pioLd32((base+REG_MIND),&temp);
	}while((temp&MIND_BUSY)&&timeout--);

}

/*
 * Read value stored in a PHY register.
 */
static uint32 vsc452mac_readphy(struct net_device *dev, uint8 phy_id, uint8 address)
{
	uint32 base = dev->base_addr;
	uint32 timeout = 200000;
	uint32 temp=0;


	pioSt32((base+REG_MADR),address | (phy_id << 8));
	pioSt32((base+REG_MCMD),0);
	pioSt32((base+REG_MCMD),1);

	do   
	{
		pioLd32( (base+REG_MIND),&temp);
	}while ( ((temp &MIND_BUSY) || (temp & MIND_NV)) && timeout-- );



	temp=0;
	pioLd32((base+REG_MRDD),&temp);
	return temp;
}




/******************************************************************
 ******************	DEVICE DRIVER ENTRY POINTS ********************
 ******************************************************************/


/*----------------------------------------------------------------
   If this driver is linked with kernel (i.e) not a module, then
   this probe function is called from Space.c
 ---------------------------------------------------------------*/
#ifndef MODULE

struct net_device * __init vsc452mac_probe(int unit)
{
	struct net_device *dev;
	int err;

	if (unit >= MAX_VSC452MAC)
		return ERR_PTR(-ENODEV);

	dev = alloc_etherdev(sizeof(struct vsc452mac_local));
	if (!dev)
		return ERR_PTR(-ENODEV);


	SET_MODULE_OWNER(dev);

	sprintf(dev->name, "eth%d", unit);
	netdev_boot_setup_check(dev);

	//dev->base_addr = (unsigned long)ioremap(vsc452mac_io[unit], 0x100);
	dev->base_addr = vsc452mac_io[unit];
	dev->irq = 3;// vsc452mac_irq[unit];
	FindMACAddress(unit,dev->dev_addr);

	err = do_vsc452mac_probe(dev);
	if (err)
		goto out;
	return dev;
out:
	free_netdev(dev);
	return ERR_PTR(err);
}
#endif


/*----------------------------------------------------------------
   If this driver is loadad as module, the module intialization
   function is called first which registers the devices
-------------------------------------------------------------*/
#ifdef MODULE


static struct net_device* dev_vsc452mac[MAX_VSC452MAC];
static int vsc452mac_count;


/*------------------------------------------------------------
 . Module initialization function
 .-------------------------------------------------------------*/
int
init_module(void)
{
	u32 port;
	u32 baseAddr;
	//u32 phy_addr;

	int i;
	struct net_device *dev;
	struct vsc452mac_local   *lp;
	int ret =0;

	for (i = 0; i < MAX_VSC452MAC; i++)
	{

		port =i;
		baseAddr = vsc452mac_io[i];
		//reset mac phy interfaces
		pioSt32 ( baseAddr + REG_MAC1, 0x0000cf00 );
		pioSt32 ( baseAddr + REG_SUPP, 0x08888 );
		pioSt32 ( baseAddr + REG_MCFG, 0x08000 );


		udelay(5000);

		//initialize mac and phy values
		pioSt32 ( baseAddr + REG_MAC1, 0 );
		pioSt32 ( baseAddr + REG_SUPP, SUPP_SPEED );
		pioSt32 ( baseAddr + REG_MCFG, 0x0018 );

		//create the device
		dev = alloc_etherdev(sizeof(struct vsc452mac_local));
		if (!dev)
	        	return -ENOMEM;

		lp = netdev_priv(dev);

		//initialize the device params
		dev->base_addr = baseAddr;
		dev->irq = 3;
		sprintf( dev->name, "eth%d",i);

		/* Setup ethernet device */
		if (do_vsc452mac_probe(dev)< 0) {
			return -1;
		}
	
#ifdef VSC452MAC_USE_DMA
		/* allocate receive buffers*/
		if ((ret = vsc452mac_dma_alloc_buffers(dev)) != 0)
		{
			printk("skb buffer allocation failed");
			//spin_unlock_irq(&lp->lock);
			//free_irq(dev->irq,dev);
			return ret;
		}

		/* dma ring and initialize */
		vsc452mac_dma_ringsetup(dev);
		vsc452mac_init_dma(dev);
#endif

	}
	/* registering reboot notifier */
	ret = register_reboot_notifier(&vsc_ethernet_notifier);
	if(ret)
       {
       	printk(KERN_ERR "vsc452_ethernet_notifier : cannot register reboot notifier (err=%d)\n", ret);
       }

	return 0;

}


/*------------------------------------------------------------
 . Cleanup when module is removed with rmmod
 .-------------------------------------------------------------*/
void
cleanup_module(void)
{
	int i;

	for (i=0; i<vsc452mac_count; i++)
	{
		struct net_device *dev = dev_vsc452mac[i];
    		struct vsc452mac_local *lp = netdev_priv(dev);

		//release_region(dev->base`_addr, 0x100);
		//iounmap(dev->base_addr);
#ifdef VSC452MAC_USE_DMA
		// free the buffers
		vsc452mac_dma_free_buffers(dev);
		// disable dma transmit and receive
		vsc452mac_disable_rx_dma(dev);
		vsc452mac_disable_tx_dma(dev);
        	dma_free_coherent(NULL,
                          (RX_MAX_DESC_COUNT*sizeof(struct dma_rx_desc_T )) +
						  (TX_MAX_DESC_COUNT*sizeof(struct dma_tx_desc_T)),
                          lp->rx_ring_desc,
                          lp->rx_ring_dma);
#endif
		unregister_netdev(dev);
		free_netdev(dev);
	}

	/* Unregister reboot notifier */
	unregister_reboot_notifier(&vsc_ethernet_notifier);


	return;
}

module_init(init_module)
module_exit(cleanup_module)



MODULE_LICENSE("GPL");


#endif /* MODULE */
