/*
 * Copyright (C) 2007 American Megatrends Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "spiflash.h"
#ifdef __UBOOT__	
//#include "soc_hw.h"			//TODO
#include <vsc452/vsc452regs.h>
#endif

#define SPI_INPUT_CLOCK 100		/* Input Frequency MHZ */
#define MAX_CLOCK	(25/2)		/* Max SPI Frequency MHZ supported by Controller */
#define MAX_READ	4

#define IO_WORD(x)	(*((volatile unsigned long *)((x))))



void 
SPIManualModeEnable(int Bank)
{
	IO_WORD(SYS_SMMC) = SYS_SMMC_MMSS | SYS_SMMC_MMCS(Bank) ;	/* High Speed and Select CS */
	IO_WORD(SYS_SMME) = SYS_SMME_MME;						/* Manual Mode Enable */
}

void 
SPIReleaseChipSelect(void)
{
	unsigned long Status;

	IO_WORD(SYS_SMMS) |= SYS_SMMS_RMMCS;					/* Release CS */

	/* Wait for Activity to Stop */
	while (1)
	{
		Status = IO_WORD(SYS_SMMS); 
		if (!(Status & SYS_SMMS_MMA))	
			break;
	//	printk("SPI is Active ...Status(0x%lx) = 0x%lx\n",SYS_SMMS,Status);	
	}

	return;
}

void 
SPIManualModeDisable(void)
{

	SPIReleaseChipSelect();
	IO_WORD(SYS_SMME) = 0x00000000;							/* Manual Mode Disable */
}

void 
SPIWriteByte(unsigned char uData)
{
	unsigned long Status;

	/* Write a Byte */
	IO_WORD(SYS_SMMDW) = (unsigned long)uData;

	/* Wait for Transfer to Complete */
	while (1)
	{
		Status  = IO_WORD(SYS_SMMS);
		if (Status & SYS_SMMS_NBR)
			break;
		//printf("WriteByte: Status (0x%lx)= 0x%lx\n",SYS_SMMS,Status);
	}
	return;
}

unsigned char 
SPIReadByte(void)
{
	unsigned long Status;

	/* Write a dummy byte for a Read */
	IO_WORD(SYS_SMMDW) = 0;

	/* Wait for Transfer to Complete */
	while (1)
	{
		Status  = IO_WORD(SYS_SMMS);
		if (Status & SYS_SMMS_NBR)
			break;
		//printf("ReadByte: Status = 0x%x\n",Status);
	}

	return (unsigned char)IO_WORD(SYS_SMMDR);
}



#if 0
#define SHOWREG8(x) printk("Register %s @ 0x%08lx = 0x%02x\n",#x,(unsigned long)&(Regs->x),Regs->x)
#define SHOWREG32(x) printk("Register %s @ 0x%08lx = 0x%08lx\n",#x,(unsigned long)&(Regs->x),Regs->x)
static
void
DumpRegisters(void)
{
	return;
}
#endif



/*****************************************************************************************************************/

static
void
InitializeCtrl(void)
{
	/* Nothing to do */
	return;
}
	


static
int
ValidateInputs(unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, int datalen)
{
	if ((cmdlen != 1) && (cmdlen != 4) && (cmdlen !=5))
	{
		printk("vsc452spi : Invalid Command Len %d\n",cmdlen);
		return -1;
	}

	if (cmd == NULL)
	{
		printk("vsc452spi : Invalid : Null Command\n");
		return -1;
	}

	if (dir != SPI_NONE)
	{
		if ( (data == NULL) || (datalen == 0))
		{
			printk("vsc452spi : Invalid data or datalen \n");
			return -1;
		}
	}
	if ((dir == SPI_READ) && (datalen > MAX_READ))
	{
		printk("vsc452spi : Invalid : Reads cannot exceed %d bytes\n",MAX_READ);
		return -1;
	}
	
	return 0;

}



#define MAX_BURST_READ 0x1000
static
int  
vsc452spi_burst_read(int bank,unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, unsigned long datalen)
{
	int i;
	
	//printf("vsc452spi_burst_read called\n");

	SPIManualModeEnable(bank);

	/* Send command and address */
	for (i=0;i<cmdlen;i++)
		SPIWriteByte(cmd[i]);

	/* Read Data */
	for(i=0;i<datalen;i++)
 		data[i] = SPIReadByte();
	SPIReleaseChipSelect();

	SPIManualModeDisable();

	//printf("vsc452spi_transfer returned\n");

	return 0;
}

static
int  
vsc452spi_transfer(int bank,unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, unsigned long datalen)
{

	int i;

	//printf("vsc452spi_transfer called\n");

	if (ValidateInputs(cmd,cmdlen,dir,data,datalen) == -1)
		return -1;

	SPIManualModeEnable(bank);

	/* Send command and address */
	for (i=0;i<cmdlen;i++)
		SPIWriteByte(cmd[i]);
		
	if (dir == SPI_READ)
	{
		for(i=0;i<datalen;i++)
 			data[i] = SPIReadByte();
	}
		
	if (dir == SPI_WRITE)
	{
		for(i=0;i<datalen;i++)
 			SPIWriteByte(data[i]);
	}


	SPIReleaseChipSelect();

	SPIManualModeDisable();

	//printf("vsc452spi_transfer returned\n");

	return 0;
}

/*****************************************************************************************************************/
#ifdef __UBOOT__	
extern void ll_serial_init(void);
static int 
vsc452spi_init(void)
{
//	ll_serial_init();
	InitializeCtrl();	
	return 0;
}
#endif


#ifndef __UBOOT__	/* linux */
struct spi_ctrl_driver vsc452spi_driver =
#else
/* 
   Uboot supports only one spi controller and has to be defined 
   in the structure "SPI_CTRL_DRIVER"
*/
struct spi_ctrl_driver SPI_CTRL_DRIVER =
#endif
{
	.name 		= "vsc452spi",
	.module 	= THIS_MODULE,
	.max_read	= MAX_READ,
	.fast_read	= 0,			/* TODO !!!! */
	.spi_transfer	= vsc452spi_transfer,
	.spi_burst_read	= vsc452spi_burst_read,
#ifdef __UBOOT__	
	.spi_init	= vsc452spi_init,
#endif
};

#ifdef __UBOOT__
void
SPI_CTRL_DRIVER_INIT(void)
{
	/* The following three lines needed for MIPS */
	SPI_CTRL_DRIVER.spi_transfer	= vsc452spi_transfer;
	SPI_CTRL_DRIVER.spi_burst_read	= vsc452spi_burst_read;
	SPI_CTRL_DRIVER.spi_init	= vsc452spi_init;

	InitializeCtrl();	
	init_MUTEX(&SPI_CTRL_DRIVER.lock);
	register_spi_ctrl_driver(&SPI_CTRL_DRIVER);
	return;
}

#else		/* Linux */

static int 
vsc452spi_init(void)
{
	InitializeCtrl();	
	init_MUTEX(&vsc452spi_driver.lock);
	register_spi_ctrl_driver(&vsc452spi_driver);
	return 0;
}


static void 
vsc452spi_exit(void)
{
	unregister_spi_ctrl_driver(&vsc452spi_driver);
	return;
}


module_init(vsc452spi_init);
module_exit(vsc452spi_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("American Megatrends Inc");
MODULE_DESCRIPTION("VSC452 SOC SPI Controller driver");

#endif
