/*
 * (C) Copyright 2008 American Megaterends
 * * This file contains the configuration parameters for the vsc452 evaluatio board 
*/

#ifndef __CONFIG_H
#define __CONFIG_H
/*---------------------------------------------------------------
 *			Board and Processor Identification
 *---------------------------------------------------------------
 */
#define CONFIG_MIPS32				1  		/* MIPS32 CPU core	*/
#define CONFIG_32BIT				1  		/* Needed for exception handling as 
							   some header files are taken from linux - To be cleaned up */

/*------------------------------------------------------------------------------------------*/
/*--------------------------------- Configurable Setting -----------------------------------*/
/*------------------------------------------------------------------------------------------*/
/* Size of U-Boot (without Env) */
#define 	CFG_MONITOR_LEN		   	CFG_PROJ_UBOOT_MONITOR_SIZE		
#define 	CFG_MAX_FLASH_BANKS		1
#define 	CONFIG_BOARD_NAME		"VSC452" 			/* Board Name 			*/


#define 	CFG_FLASH_BASE          	CFG_PROJ_FLASH_START		/* Should be 0xBFC00000 */
#define 	CFG_SDRAM_LEN			CFG_PROJ_MEMORY_SIZE 		/* SDRAM Size */
#define 	CFG_FLASH_SIZE			CFG_PROJ_FLASH_SIZE		/* Flash Size*/
#define 	CFG_FLASH_SECTOR_SIZE		CFG_PROJ_ERASE_BLOCK_SIZE	/* Flash Sector Size	*/

#define 	CONFIG_ETHADDR			00:00:00:00:00:00		/* Ethernet MAC Address 		*/
#define 	CONFIG_ETH1ADDR			00:00:00:00:00:00		/* Ethernet1 MAC Address 		*/
#define 	CONFIG_MENUKEY		 	27				/* ESC Key to enter menu		*/
#define 	CONFIG_SILENT_CONSOLE		1

/* Used for Linux Booting */
#define 	CONFIG_MEMSIZE_IN_BYTES
#define 	CONFIG_BOOTARGS			"console=ttyS0,9600 root=/dev/mtdblock7 rw rootfstype=jffs2 init=/sbin/netapp_init"
#define 	CONFIG_BOOTARGS2		"console=ttyS0,9600 root=/dev/mtdblock9 rw rootfstype=jffs2 init=/sbin/netapp_init"
	
/* Various clocks derived from . Board specific */
#define CLOCK_TICK_RATE				(300*1000*1000)
#define CFG_CP0_COUNT_RATE			(CLOCK_TICK_RATE/2)
#define CFG_HZ					1000


/* Default SPI Settings - Eval Board SPI does not support Read ID */
#define CONFIG_DEFAULT_SPI_NAME 			"ST Micro m25p40"
#define CONFIG_DEFAULT_SPI_SIZE 			CFG_PROJ_FLASH_SIZE
#define CONFIG_DEFAULT_SPI_ERASE_SIZE			CFG_PROJ_ERASE_BLOCK_SIZE


/*---------------------------------------------------------------
 *  			Default Memory Test Range
 *---------------------------------------------------------------
 */
#define CFG_MEMTEST_START			0x81000000
#define CFG_MEMTEST_END				CFG_MEMTEST_START + (1024*1024*16)

 

/*---------------------------------------------------------------
 *	             Serial Port Configuration
 *---------------------------------------------------------------
 */
#define CFG_BAUDRATE_TABLE      { 9600, 19200, 38400, 57600, 115200 }


/*---------------------------------------------------------------
 *	           Required DHCP Vendor Extensions
 *---------------------------------------------------------------
 */
#define CONFIG_BOOTP_MASK 	(CONFIG_BOOTP_SUBNETMASK | \
					   CONFIG_BOOTP_GATEWAY    | \
					   CONFIG_BOOTP_HOSTNAME)

/*---------------------------------------------------------------
 *	               Supported Commands
 *---------------------------------------------------------------
 */
#define CONFIG_COMMANDS		\
	(CFG_CMD_EEPROM|CFG_CMD_DHCP|CFG_CMD_NET| CFG_CMD_PING|CFG_CMD_MII|	\
	CFG_CMD_MEMORY|CFG_CMD_CONSOLE|CFG_CMD_FLASH|		\
	CFG_CMD_AUTOSCRIPT|CFG_CMD_RUN|CFG_CMD_BDI|		\
	CFG_CMD_ELF|CFG_CMD_IMLS|CFG_CMD_IMI|			\
	CFG_CMD_RUN|CFG_CMD_SAVEENV |CFG_CMD_ASKENV |		\
	CFG_CMD_ENV |	CFG_CMD_NAND |				\
	CFG_CMD_IRQ | CFG_CMD_MISC | CFG_CMD_LOADB)

/* CFG_CMD_DATE  | CFG_CMD_SNTP */





/* IMPORTANT: This must be included AFTER the definition of
               CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

/*---------------------------------------------------------------
 * 				 Default Environment Values
 *--------------------------------------------------------------*/
#define CONFIG_BAUDRATE        			9600
#define CONFIG_BOOTDELAY        		1
#define CFG_AUTOLOAD				"no"
#define CONFIG_LOADS_ECHO			1
#define CFG_LOADS_BAUD_CHANGE			1
#define CFG_LOAD_ADDR				0x80500000
#define CONFIG_BOOTCOMMAND			"nboot"
//#define CONFIG_EXTRA_ENV_SETTINGS "menucmd=bootmenu\0silent=1"
#define CONFIG_EXTRA_ENV_SETTINGS 		"menucmd=bootmenu\0autostart=yes\0bootdevice=0\0image=primary"
#define CFG_REDUNDAND_ENVIRONMENT		1

/*---------------------------------------------------------------
 * 				Network Configuration
 *--------------------------------------------------------------*/
#define CONFIG_MII				1
#define CONFIG_OVERWRITE_ETHADDR_ONCE		1
#define CONFIG_NET_RETRY_COUNT			25
#define CONFIG_PHY_CMD_DELAY			1
//#define CONFIG_UDP_CHECKSUM  

/*---------------------------------------------------------------
 * Miscellaneous configurable options
 *---------------------------------------------------------------*/
#define CONFIG_BOARD_PRE_INIT			1

#define CFG_LONGHELP				1			/* Long Help    		*/
#define CFG_PROMPT				CONFIG_BOARD_NAME">"	/* Monitor Command Prompt	*/
#define CFG_PBSIZE				(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)
									/* Print Buffer Size 		*/
#define CFG_MAXARGS				16			/* Max number of command args	*/
#define CFG_BARGSIZE				CFG_CBSIZE		/* Boot Argument Buffer Size 	*/
#define CFG_EXTBDINFO				1			/* Extended Board Info 		*/
//#define CFG_ALT_MEMTEST			1			/* Allow Extended Memory Test 	*/
#define CFG_DEVICE_NULLDEV			1			/* Enable Null Device 		*/
#define CFG_CONSOLE_INFO_QUIET 			1
#define CFG_DIRECT_FLASH_TFTP			1			/* Allow TFTP download to flash	*/

/* Console I/O Buffer Size */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CBSIZE				1024
#else
#define CFG_CBSIZE				256
#endif

#define CONFIG_USE_IRQ				1			/* U-Boot can use IRQ */
#define CONFIG_PASS_PLATFORM_ID                 1


/*-----------------------------------------------------------------
 * Stack sizes
 *
 *-----------------------------------------------------------------
 */
#define CFG_BOOTPARAMS_LEN			128*1024
#define CFG_INIT_SP_OFFSET			0x400000	/* Initial Stack offset*/

/*---------------------------------------------------------------
 * NAND Flash
 *----------------------------------------------------------------
 */
#undef CFG_NAND_LEGACY

#define CFG_NAND0_BASE		0xB0200040
#undef CFG_NAND1_BASE

#define CFG_NAND_BASE_LIST	{ CFG_NAND0_BASE }
#define CFG_MAX_NAND_DEVICE	1	/* Max number of NAND devices */
#define NAND_MAX_CHIPS		1


/*---------------------------------------------------------------
 * EEPROM 
 *----------------------------------------------------------------
 */
#define CFG_I2C_EEPROM_ADDR     	0x50
#define CFG_I2C_EEPROM_ADDR_LEN 	2
#define CONFIG_HARD_I2C         	1
#define CFG_I2C_SLAVE           	CFG_I2C_EEPROM_ADDR
#define CFG_I2C_SPEED           	100000
#define CFG_MAC_ADDR_OFFSET		71		
#define CFG_MAC_ADDR1_OFFSET		94 
#define CFG_READ_MAC_FROM_FRU_SUPPORT	1
#define CFG_CONTROLLER_BUS_NO		0           	
/*----------------------------------------------------------------
 *             Physical Memory Map of SDRAM and Flash
 *----------------------------------------------------------------
 */
#define CFG_SDRAM_BASE 				CFG_PROJ_MEMORY_START

/* Cannot use SPD to detect SDRAM Information*/
#undef CONFIG_SPD_EEPROM

#ifdef CFG_PROJ_FLASH_TYPE_NOR
	/* Use CFI Driver to detect the Flash Information */
	#define CFG_FLASH_CFI    		1
	#define CFG_FLASH_CFI_DRIVER    	1
	#define CFG_ENV_IS_IN_FLASH 
#endif

#ifdef CFG_PROJ_FLASH_TYPE_SPI
	/* Use SPI Driver to detect the Flash Information */
	#define CONFIG_HAS_SPI			1
	#define CFG_FLASH_SPI_DRIVER    	1
	#define CFG_ENV_IS_IN_SPI		1
#endif

/* Max number of sectors on one chip*/
#define CFG_MAX_FLASH_SECT      		512
//#define CFG_MAX_FLASH_SECT	(CFG_FLASH_SIZE/CFG_FLASH_SECTOR_SIZE)

/*--------------------------------------------------------------
 *					Monitor Information
 *--------------------------------------------------------------
*/
//#define CFG_MONITOR_BASE	   		CFG_FLASH_BASE
// CFG_MONITOR_BASE Should Match with CFG_FLASH_BASE
#define	CFG_MONITOR_BASE			TEXT_BASE

/*---------------------------------------------------------------
 *                    Environment organization
 *---------------------------------------------------------------
 */
#define CONFIG_ENV_OVERWRITE 			1
#define CFG_ENV_SECT_SIZE	 		CFG_PROJ_UBOOT_ENV_SIZE

/*
 * Adjust Total Size of Environment so that it fits in a complete sector
 * along with the variable "env_size". Actual Usable environment Size is
 * CFG_ENV_SECT_SIZE - 2 * sizeof(unsigned long). The unsigned longs are
 * "env_size" and crc32
 */
#define CFG_ENV_SIZE 				(CFG_ENV_SECT_SIZE - 4)

#ifdef CFG_PROJ_UBOOT_ENV_START
#define CFG_ENV_ADDR				CFG_PROJ_UBOOT_ENV_START
#else
#define CFG_ENV_ADDR				(CFG_FLASH_BASE+CFG_MONITOR_LEN)
#endif

#ifdef CFG_PROJ_UBOOT_ENV2_START
#define CFG_ENV2_ADDR				CFG_PROJ_UBOOT_ENV2_START
#endif
		
/*----------------------------------------------------------------
 *               Size of malloc() pool
 *---------------------------------------------------------------
 */
#define CFG_MALLOC_LEN	  			((CFG_ENV_SIZE+4) + (128 *1024))


/*----------------------------------------------------------
 * Cache Configuration
 *----------------------------------------------------------
 */
#define CFG_DCACHE_SIZE				16384
#define CFG_ICACHE_SIZE				16384
#define CFG_CACHELINE_SIZE			16

/*----------------------------------------------------------
 * Board Specific Configs
 *----------------------------------------------------------
 */
#define CONFIG_EXPORT_ETH_FNS			1

/*----------------------------------------------------------
 * YAFU support
 *----------------------------------------------------------
 */
//#define CFG_YAFU_SUPPORT 			1


#endif							/* __CONFIG_H */
